/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "breadcrumb.h"

#include <QPainter>
#include <QtDebug>
#include <QStyledItemDelegate>
#include <QPersistentModelIndex>

#include <QStandardItem>
#include <QStandardItemModel>

//! --------------------- BreadCrumb -------------------------------------------
BreadCrumb::BreadCrumb(QWidget * parent) : QWidget(parent)
{
    _model     = 0;
    _delegate  = new QStyledItemDelegate(this);
        
    QStyleOption opt;
    opt.initFrom(this);
    
    int isize = style()->pixelMetric(QStyle::PM_SmallIconSize,&opt,this);
    _iconSize  = QSize(isize,isize);
    
    setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
}

//! --------------------- model method -----------------------------------------
void BreadCrumb::setModel(MenuModel * m) 
{
    if (m == _model) return;
    if (_model && _model->parent() == this) {
        delete _model;
        // disconnect signals
    }
    _model = m;
    setRootIndex(_model->index(0,0,QModelIndex()));
    setCurrentIndex(_model->index(0,0,QModelIndex()));
    // connect signals
    
    // update
    //update();
}

//! --------------------- restore & save method --------------------------------
void BreadCrumb::restorePath(QString itemText)
{
  qDebug() << "BreadCrumb --> restorePath itemText" << itemText; 
  QModelIndex idx = _model->IndexFromItemText(itemText);
  setCurrentIndex(idx);
}


//! --------------------- pathList ---------------------------------------------
QList<QModelIndex> BreadCrumb::pathList() const 
{
    QList<QModelIndex> path;
    QModelIndex iterator = _currentIndex;
    QModelIndex root = _rootIndex;
    while(iterator!=root && iterator.isValid()){
        path.prepend(iterator);
        iterator = iterator.parent();
    }
  return path;
}


//! --------------------- paintEvent -------------------------------------------
void BreadCrumb::paintEvent(QPaintEvent * event) 
{
    QPainter painter(this);
    QList<QModelIndex> toBePainted = pathList();
    //qDebug() << "BreadCrumb --> Paint Event toBePainted:" << toBePainted; 

    QModelIndex iterator = currentIndex();
    QRect r = contentsRect();
    QRect bounding;
    for(int i=0;i<toBePainted.count();i++){
        const QModelIndex &drawing = toBePainted[i];
        QString text = drawing.data(Qt::DisplayRole).toString();

        QStyleOptionViewItemV2 option;
        option.initFrom(this);

        option.decorationAlignment = Qt::AlignLeft|Qt::AlignVCenter;
        option.displayAlignment = Qt::AlignLeft|Qt::AlignVCenter;
        option.decorationSize = iconSize();
        QSize hint = _delegate->sizeHint(option, drawing);

        option.rect = QRect(r.left(), r.top(), hint.width(), hint.height());
        _delegate->paint( &painter, option, drawing );
        r.setLeft(r.left()+hint.width()+4);
        
        //! paint arrow
        if(i != toBePainted.count() -1){
          QStyleOption opt;
          opt.initFrom(this);
          opt.rect = QRect(r.left(), r.top(), 16, hint.height());
          style()->drawPrimitive(QStyle::PE_IndicatorArrowRight, &opt, &painter, this);
          r.setLeft(r.left()+20);
        }
    }
}

QModelIndex BreadCrumb::rootIndex() const 
{
    return _rootIndex;
}

void BreadCrumb::setRootIndex(const QModelIndex &i) 
{
    if (_rootIndex == i) return;
    _rootIndex = i;
    update();
    qDebug() << "BreadCrumb setRootIndex" << i;     
}

void BreadCrumb::setCurrentIndex(const QModelIndex &i) 
{
    if (_currentIndex == i) return;
    _currentIndex = i;
    update();
    qDebug() << "BreadCrumb setCurrentIndex" << i; 
}

QModelIndex BreadCrumb::currentIndex() const 
{
    return _currentIndex;
}

QSize BreadCrumb::sizeHint() const
{
    int l, t, r, b;
    getContentsMargins(&l, &t, &r, &b);
    return QSize(l+r+400, t+b+qMax(fontMetrics().height(), iconSize().height()));
}



void BreadCrumb::setIconSize(const QSize & s)
{
    if(_iconSize == s) return;
    _iconSize = s;
    updateGeometry();
    update();
}

QSize BreadCrumb::iconSize() const
{
    return _iconSize;
}

QRect BreadCrumb::visualRect(const QModelIndex & index) const
{
    return QRect();
}
