/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef _CENTRAL_TOOLBAR_H_
#define _CENTRAL_TOOLBAR_H_

#include "Widget/searchfield.h"
#include "CentralWidget/mainmode.h"
#include "CentralWidget/menumodel.h"
#include "CentralWidget/breadcrumb.h"

#include <QtGui>

class CentralToolBar;

namespace GLOBAL
{
    CentralToolBar* centralToolBar();
}



//! ---------- class CentralToolBar --------------------------------------------
class CentralToolBar : public QToolBar
{
Q_OBJECT
  static CentralToolBar* global_instance;

  public:
    CentralToolBar(QWidget *parent);
    //! return global instance    
    static CentralToolBar* instance() { return global_instance; }
    
    void updateMode(Main::MainMode mode);
    void updateCurrentIndex(const QModelIndex &idx);
    
    BreadCrumb         *_breadcrumb;
    
  private:
    MenuModel          *_menuModel;
    SearchField        *_collectionSearch;
    SearchField        *_playlistSearch;
    SearchField        *_radioSearch;
    QAction            *goUpAct;
    QAction            *goDownAct;
    QAction            *_actCSearch;
    QAction            *_actPSearch;
    QAction            *_actRSearch;
    
    int                _currentBrowserIdx;
    QStringList        _browserList;
        
  private slots:
     void sendCollectionSearch();
     void sendPlaylistSearch();
     void sendRadioSearch();
     void activePrev();
     void activeNext();

  signals:
    void collectionSearch(QString);
    void playlistSearch(QString);
    void radioSearch(QString);
    void itemActivated(QModelIndex);
};

#endif // _CENTRAL_TOOLBAR_H_
