/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "menumodel.h"
#include "menuitem.h"
#include "constants.h"


#include <QDebug>
#include <QDir>
#include <QMap>
#include <QString>
#include <QStringList>
#include <QAction>


namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}


//! ---------- global instance -------------------------------------------------
MenuModel* MenuModel::global_instance = 0;

namespace GLOBAL
{
    MenuModel* menuModel()
    {
        return MenuModel::instance();
    }
}


//! ----------- class MenuModel ------------------------------------------------
MenuModel::MenuModel(QObject *parent) : QStandardItemModel(parent)
{
   global_instance = this;

   createActions();
   populate(); 
}


MenuModel::~MenuModel()
{
  
}

//! ----------- create Actions -------------------------------------------------
void MenuModel::createActions()
{
  //! Collection Views Actions  
  QAction *albumviewAct    = new QAction(this); 
  QAction *artistviewAct   = new QAction(this); 
  QAction *trackviewAct    = new QAction(this); 
  QAction *genreviewAct    = new QAction(this); 
  QAction *favoriteviewAct = new QAction(this); 

  //! Radio Views Actions
  QAction *icecastviewAct  = new QAction(this); 
  QAction *shoutcastviewAct = new QAction(this); 
  QAction *customRadioviewAct = new QAction(this); 
  QAction *favoriteRadioviewAct = new QAction(this); 

  //! Playlist Views Actions
  QAction *playlistviewAct = new QAction(this); 
  QAction *playlistdetailviewAct = new QAction(this); 

  //! Filer Browser Actions
  QAction *filerAction = new QAction(this); 

  //! Global actions set up
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();  
  actions->insert("albumview", albumviewAct);  
  actions->insert("artistview", artistviewAct);
  actions->insert("trackview", trackviewAct);
  actions->insert("genreview", genreviewAct);
  actions->insert("favoriteview", favoriteviewAct);
  actions->insert("icecastview", icecastviewAct);  
  actions->insert("shoutcastview", shoutcastviewAct);  
  actions->insert("customradioview", customRadioviewAct);
  actions->insert("favoriteradioview", favoriteRadioviewAct);  
  actions->insert("playlistview", playlistviewAct);  
  actions->insert("playlistdetailview", playlistdetailviewAct);  
  actions->insert("fileBrowser", filerAction);      
}

//! ----------- model population -----------------------------------------------
void MenuModel::populate()
{
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();  

    
  QStandardItem *rootItem = this->invisibleRootItem();
  MenuItem* homeItem = new MenuItem(tr("Home"), QIcon(":/images/go-home.png"), MenuHomeType);
  rootItem->appendRow(homeItem);
  
  //! populate Home Item
  MenuItem* item2 = new MenuItem(tr("Music browser"), QIcon(":/images/collection.png"), MenuFirstLevelType);
  homeItem->appendRow(item2);

  MenuItem* item3 = new MenuItem(tr("Playlist browser"), QIcon(":/images/playlist-file-48x48.png"), MenuFirstLevelType);
  homeItem->appendRow(item3);

  _radioItem = new MenuItem(tr("Radio browser"), QIcon(":/images/podcast110x110.png"), MenuFirstLevelType);
  homeItem->appendRow(_radioItem);

  MenuItem* item5 = new MenuItem(tr("File browser"), QIcon(":/images/folder.png"), MenuFilerType);
  item5->setData(QVariant::fromValue(actions->value("fileBrowser")), ActionRole);
  homeItem->appendRow(item5);
      
  //! populate Collection Item
  MenuItem* item21 = new MenuItem(tr("album view"), QIcon(":/images/covericon.png"), MenuCollectionType); 
  item21->setData(QVariant::fromValue(actions->value("albumview")), ActionRole);
  item2->appendRow(item21);
 
  MenuItem* item22 = new MenuItem(tr("artist view"), QIcon(":/images/view-artist.png"), MenuCollectionType); 
  item22->setData(QVariant::fromValue(actions->value("artistview")), ActionRole);
  item2->appendRow(item22);

  MenuItem* item23 = new MenuItem(tr("track view"), QIcon(":/images/playlist-addfile.png"), MenuCollectionType); 
  item23->setData(QVariant::fromValue(actions->value("trackview")), ActionRole);
  item2->appendRow(item23);

  MenuItem* item24 = new MenuItem(tr("genre view"), QIcon(":/images/genre.png"), MenuCollectionType); 
  item24->setData(QVariant::fromValue(actions->value("genreview")), ActionRole);
  item2->appendRow(item24);

  MenuItem* item25 = new MenuItem(tr("favorite view"), QIcon(":/images/view-favorite.png"), MenuCollectionType); 
  item25->setData(QVariant::fromValue(actions->value("favoriteview")), ActionRole);
  item2->appendRow(item25);

  //! populate Paylist Browser Item
  MenuItem* item31 = new MenuItem(tr("playlist view"), QIcon(":/images/playlist-file-48x48.png"), MenuPlaylistType); 
  item31->setData(QVariant::fromValue(actions->value("playlistview")), ActionRole);
  item3->appendRow(item31);

  MenuItem* item32 = new MenuItem(tr("track view"), QIcon(":/images/playlist-addfile.png"), MenuPlaylistType); 
  item32->setData(QVariant::fromValue(actions->value("playlistdetailview")), ActionRole);
  item3->appendRow(item32);

  //! populate Radio Browser Item
  populateRadioItem();
}




//! --------- populateUserRadioFile --------------------------------------------
void MenuModel::populateRadioItem() 
{
    QMap<QString, QAction*> *actions = GLOBAL::globalActions();

    //! removeRows for radio Item
    _radioItem->removeRows(0, _radioItem->rowCount() ); 

    //! add IceCast & ShoutCast
    MenuItem *mi1 = new MenuItem(tr("iceCast stream"), QIcon(":/images/icecast.png"), MenuRadioType);
    mi1->setData(QVariant::fromValue(actions->value("icecastview")), ActionRole);
    _radioItem->appendRow(mi1);

    MenuItem *mi2 = new MenuItem(tr("shoutCast stream"), QIcon(":/images/shoutcast.png"), MenuRadioType);
    mi2->setData(QVariant::fromValue(actions->value("shoutcastview")), ActionRole);
    _radioItem->appendRow(mi2);
  
    //! search for xspf custom radio file 
    const QStringList filters = QStringList() << "*.xspf";
    QDir sourceDir(_CONFIGDIR + "/radio");
    sourceDir.setNameFilters(filters);  

    QStringList entryList = sourceDir.entryList(filters, QDir::Files, QDir::Name);
    foreach (const QString &file, entryList) 
    {
      qDebug() << " ---- MenuModel --> populateUserRadioFile : custom radio file  " << file;

      MenuItem *mi = new MenuItem(QFileInfo(file).baseName(),
                                  QIcon(":/images/podcast110x110.png"), 
                                  MenuCustomRadioType);
      mi->setData(QVariant::fromValue(actions->value("customradioview")), ActionRole);
      mi->setData(QVariant(file), FileRole);
      _radioItem->appendRow(mi);
    }
    
    //! add Favorite
    MenuItem *mi3 = new MenuItem(tr("favorite stream"), QIcon(":/images/view-favorite.png"), MenuRadioType);
    mi3->setData(QVariant::fromValue(actions->value("favoriteradioview")), ActionRole);
    _radioItem->appendRow(mi3);    
}



void MenuModel::update()
{ 
    populateRadioItem();
} 


QModelIndex MenuModel:: IndexFromItemText(QString text)
{ 
  QList<QStandardItem *>  list = this->findItems  ( text, Qt::MatchRecursive, 0); 

  if(list.isEmpty()) 
    return this->indexFromItem(this->invisibleRootItem()->child(0));

  QModelIndex idx = this->indexFromItem ( list.first() );
  
  if(!idx.isValid())
    idx = this->indexFromItem(this->invisibleRootItem()->child(0));

  return idx;
} 
