/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "menuview.h"
#include "menumodel.h"


//! -------- Class MenuView ----------------------------------------------------
MenuView::MenuView(MenuModel *model, QWidget *parent) : QTreeView(parent)
{
    //! config QTreeView
    this->header()->hide();
    this->setWordWrap(true);
    this->setSelectionMode(QAbstractItemView::NoSelection);
    this->setRootIsDecorated(false);

    this->setItemDelegate(new MenuViewDelegate(model, this));

    //connect(this, SIGNAL(doubleClicked(QModelIndex)),this, SLOT(itemClicked(QModelIndex)));
    //connect(this, SIGNAL(clicked(QModelIndex)),this, SIGNAL(itemActivated(QModelIndex)));
       qDebug() << "--- MenuView::MenuView(QWidget *parent)";
    this->setModel(model); 

    this->expandAll();
    this->setContentsMargins(5,5,5,5);

}


//! --------- itemClicked ------------------------------------------------------
void MenuView::itemClicked(QModelIndex idx)
{
   qDebug() << "--- MenuView::itemClicked idx " << idx;
   QStandardItem *item = static_cast<QStandardItemModel*>( this->model() )->itemFromIndex( idx );
   
   //! TEST 
   // a voir si je ne peux aps faire if(item->rowCount() == 0) on continue
   //! FIN TEST
   MenuItem *mi = dynamic_cast<MenuItem*>(item);
        
   qDebug() << "--- MenuView::itemClicked item " << mi->data(Qt::DisplayRole).toString();
   QString file;
   QAction *a;
   switch(mi->type()) 
   {
     case MenuCustomRadioType : 
       file =  idx.data(FileRole).toString();
       qDebug() << "--- MenuView::itemClicked custom radio file " << file;
     case MenuRadioType : 
     case MenuPlaylistType :   
     case MenuCollectionType : 
     case MenuFilerType:
       a = idx.data(ActionRole).value<QAction*>();
       qDebug() << "--- MenuView::itemClicked action = " << a;
       
       emit itemActivated(idx);
       break;
     
     default: break;
   }
}
         
         
         
//! --------- drawBranches -----------------------------------------------------
void MenuView::drawBranches ( QPainter * painter, const QRect & rect, const QModelIndex & index ) const
{
  Q_UNUSED(painter)
  Q_UNUSED(rect)
  Q_UNUSED(index)
  // draw nothing
}


//!------------MenuViewDelegate ------------------------------------------------
MenuViewDelegate::MenuViewDelegate(MenuModel *model, QObject *parent):QItemDelegate(parent)
{
  _model = model;
}


void MenuViewDelegate::paint ( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const
{
 
    //!QStyleOptionViewItem opt = option;
    //!opt.rect.adjust(-10,0,0,0);
    //!QItemDelegate::paint(painter, opt, index);
    

    QStandardItem *item = static_cast<QStandardItemModel*>( _model )->itemFromIndex( index );
    MenuItem *mi = dynamic_cast<MenuItem*>(item);
    QSize iconSize;
    
    //QRect itemRect(option.rect.adjusted(-10,10,0,0));
    QRect itemRect(option.rect);
    if (mi->type() == MenuFirstLevelType || mi->type() == MenuFilerType) {
      painter->setFont(QFont("Arial", 9, QFont::Bold));
      iconSize = QSize(22,22);
    }
    else {
      painter->setFont(QFont("Arial", 9, QFont::Normal));
      iconSize = QSize(18,18);
    }

     QFontMetrics fm = QFontMetrics(painter->font());

    //! paint icon
    QVariant value = index.data(Qt::DecorationRole);
    QIcon icon = qvariant_cast<QIcon>(value);
    
    QRect decorationRect = QRect(itemRect.topLeft(),iconSize);
    icon.paint(painter, decorationRect.adjusted(-15 ,10,0,8), option.decorationAlignment);

    //! paint text
    QString text = index.data(Qt::DisplayRole).toString();
    //painter->drawText(QPoint(itemRect.left() + 24,itemRect.top()+fm.height()),text);
    painter->drawText(itemRect.adjusted(20 ,10,0,0),Qt::AlignVCenter, text);

}

QSize MenuViewDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    //Q_UNUSED(index)
  
 //! TEST pour supprimer le premier item 
 if(!index.isValid()) return QSize();
    
    QStandardItem *item = static_cast<QStandardItemModel*>( _model )->itemFromIndex( index );
    MenuItem *mi = dynamic_cast<MenuItem*>(item);
    if(mi->type() == MenuHomeType)
        return QSize(0, 0);
    else if (mi->type() == MenuFirstLevelType || mi->type() == MenuFilerType)
        return QSize(option.rect.width(), 32);
    else 
      return QSize(option.rect.width(), 26);
}
