/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _COLLECTION_MODEL_H
#define _COLLECTION_MODEL_H

#include "CollectionModel/modelitem.h"

#include <QAbstractItemModel>
#include <QStandardItem>
#include <QModelIndex>
#include <QVariant>

//! ----- CollectionModel ------------------------------------------------------
class CollectionModel;

namespace GLOBAL
{
    CollectionModel* collectionModel();
}


class CollectionModel : public QAbstractItemModel
{
Q_OBJECT
    static CollectionModel* globalCollectionModel;

 public:
     //CollectionModel(const QStringList &headers, QObject *parent = 0);
     CollectionModel(QObject *parent = 0);
     ~CollectionModel();

     //! return global instance
     static CollectionModel* instance() { return globalCollectionModel; }    
    
     //! QAbstractItemModel subclass
     QVariant data(const QModelIndex &index, int role) const;
     QVariant headerData(int section, Qt::Orientation orientation = Qt::Horizontal, int role = Qt::DisplayRole) const;
     QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;
     QModelIndex parent(const QModelIndex &index) const;
     int rowCount(const QModelIndex &parent = QModelIndex()) const;
     int columnCount(const QModelIndex &parent = QModelIndex()) const;
     Qt::ItemFlags flags(const QModelIndex &index) const;
     bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole);
     bool setHeaderData(int section, Qt::Orientation orientation, const QVariant &value, int role = Qt::EditRole);
     
     bool insertRows(int position, int rows, const QModelIndex &parent = QModelIndex());
     bool removeRows(int position, int rows,const QModelIndex &parent = QModelIndex());
    
     QModelIndex addChild(const QModelIndex &parent, const QVector<QVariant> &data);
     
     void clear();
     ModelItem *getItem(const QModelIndex &index) const;

     //! get Track method
     QStringList getItemChildrenTracks(const QModelIndex &itemIdx);
     QStringList getAlbumChildrenTracksGenre(const QModelIndex &albumIdx, QString genre);

     //! filtering method
     //bool isFiltered(const QModelIndex &itemIdx);
     bool isTrackFiltered(const QModelIndex &trackIdx);
     bool isAlbumFiltered(const QModelIndex &albumIdx);
     bool isArtistFiltered(const QModelIndex &artistIdx);
     bool isArtistGenreFiltered(const QModelIndex &artistIdx, QString genre);
     bool isAlbumGenreFiltered(const QModelIndex &albumIdx, QString genre);
     QString                      _filterPattern;
     
     //! genre method
     bool isArtistGenre(const QModelIndex &artistIdx, QString genre);
     bool isAlbumGenre(const QModelIndex &albumIdx, QString genre);
     QStringList                  _listGenre;
     
     //! playing status method
     bool itemIsPlaying(const QModelIndex &index);
     void setItemIsPlaying(const QModelIndex &index, bool playing);
     
     QModelIndex getPlayingTrackItem();
     QString getPlayingTrackItemGenre();
     void updatePlayingItem(const QString filename);
     void stopPlayingItem();

     //! Favorites status method
     bool itemIsFavorite(const QModelIndex &Idx);
     void setItemIsFavorite(const QModelIndex &Idx, bool isfavorite);
     
 private:
     ModelItem          *_rootItem;
     QModelIndex        _playingItemIdx;
     
  signals:
    void signalPlayingStatusChanged();
    void signalFavoriteStatusChanged();    
    void modelCleared();
 };


#endif // _COLLECTION_MODEL_H
