/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionModel/collectionpopulator.h"
#include "CollectionModel/collectionmodel.h"
#include "Database/databasemanager.h"

#include <QFileInfo>
#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>
#include <QVariant>
#include <QDebug>

CollectionPopulator::CollectionPopulator()
{
  _collectionModel = CollectionModel::instance();
  _player          = Player::instance();
  _isRunning       = false;
}

void CollectionPopulator::run()
{
    qDebug() << " --- CollectionPopulator--> Starting model population:";

    DatabaseManager dbManager("populateModel");
    if (!dbManager.connectToDB()) {
        _isRunning = false;
        return;
    }
    
    _collectionModel->clear();

    QModelIndex albumsParent;
    QModelIndex tracksParent;
    QString     genre;
    {
      QSqlQuery artistsQuery("SELECT id,interpret,favorite FROM interprets ORDER BY interpret ASC",
                             *dbManager.sqlDb());
      while (artistsQuery.next()) 
      {
          QVector<QVariant> artistdata ;
          artistdata.append( artistsQuery.value(0).toInt() );      //! 1-artist id
          artistdata.append( artistsQuery.value(1).toString() );   //! 2-artist name
          artistdata.append( QStringList());                       //! 3-artist cover path list

          albumsParent = _collectionModel->addChild(QModelIndex(),artistdata);
          bool isArtistFavorite = artistsQuery.value(2).toInt();
          if(isArtistFavorite)
              _collectionModel->setItemIsFavorite(albumsParent, isArtistFavorite);       
		  
          QSqlQuery albumsQuery("SELECT id,album,year,cover,favorite FROM albums WHERE id IN (SELECT album FROM tracks WHERE interpret="+artistsQuery.value(0).toString()+") ORDER BY year ASC;",
                                          *dbManager.sqlDb());
					  
          while (albumsQuery.next()) 
          {
              QVector<QVariant> albumdata;
              albumdata.append( albumsQuery.value(0).toInt());     //! 1-album id
              albumdata.append( albumsQuery.value(1).toString());  //! 2-album name
              albumdata.append( albumsQuery.value(2).toString());  //! 3-album year
              albumdata.append( albumsQuery.value(3).toString());  //! 4-album cover path
  
              tracksParent = _collectionModel->addChild(albumsParent,albumdata);
              bool isAlbumFavorite = albumsQuery.value(4).toInt();
              if(isAlbumFavorite)
                  _collectionModel->setItemIsFavorite(tracksParent, isAlbumFavorite);       

              QSqlQuery tracksQuery("SELECT id,trackname,filename,track,genre FROM view_tracks WHERE albumID="+albumsQuery.value(0).toString()+" AND interpretID="+artistsQuery.value(0).toString()+" ORDER BY track ASC;",
                                              *dbManager.sqlDb());
              while (tracksQuery.next()) 
              {
                  genre = tracksQuery.value(4).toString();
                  if(genre.isEmpty()) genre = "unknown genre";

                  QVector<QVariant> trackdata ;
                  trackdata.append( tracksQuery.value(0).toInt());    //! 1-track id
                  trackdata.append( tracksQuery.value(1).toString()); //! 2-track name
                  trackdata.append( tracksQuery.value(2).toString()); //! 3-filename
                  trackdata.append( tracksQuery.value(3).toString()); //! 4-track number
                  trackdata.append(genre);                            //! 5-genre
		      
                  //! mise à jour de la liste des genres dans le modele
                  if (!_collectionModel->_listGenre.contains(genre) ) {
                      _collectionModel->_listGenre.append(genre);
                      //qDebug() << " ---- Model Populator : adding genre :" << genre;
                  }		      
                  _collectionModel->addChild(tracksParent,trackdata);
		      
              } //! fin while track
          } //! fin while album
      } //! Fin while artist
	  }
    
    //! Update of playing item status with current track playing from Player 
    QString currentTrack = _player->currentMediaSource();
    if(!currentTrack.isEmpty()) {
        _collectionModel->updatePlayingItem(QFileInfo(currentTrack).absoluteFilePath());
    }
    qDebug() << " --- CollectionPopulator -->Ending model population:";

    _isRunning = false;
    emit modelPopulated();
}
