/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionModel/modelitem.h"
#include <QDebug>


//! ----- ModelItem ------------------------------------------------------------
ModelItem::ModelItem(const QVector<QVariant> &data, ModelItem *parent)
{
    _parentItem = parent;
    _itemData = data;
}

ModelItem::~ModelItem()
{
    qDeleteAll(_childItems.begin(),_childItems.end());
    _childItems.clear();
}


//! ----- Data methods ---------------------------------------------------------
// setData(int column, const QVariant &value)
// setData(const QVector<QVariant> &data)
// getData(int column)
bool ModelItem::setData(int column, const QVariant &value)
{
    if (column < 0 || column >= _itemData.size())
        return false;

    _itemData[column] = value;
    return true;
}

void ModelItem::setData(const QVector<QVariant> &data)
{
  _itemData = data;
}

     
QVariant ModelItem::getData(int column) const
{
    if ((column < 0) || (column > _itemData.size()))
        return QVariant();

    return _itemData.value(column);
}

int ModelItem::columnCount() const
{
    return _itemData.count();
}

//! ----- Parent methods -------------------------------------------------------
ModelItem *ModelItem::parent()
{
    return _parentItem;
}


//! ----- Child methods --------------------------------------------------------
ModelItem *ModelItem::child(int index)
{
    if (index > _childItems.count())
        return NULL;

    return _childItems.value(index);
}


int ModelItem::childCount() const
{
    return _childItems.count();
}


int ModelItem::childNumber() const
{
    if (_parentItem)
        return _parentItem->_childItems.indexOf(const_cast<ModelItem*>(this));

    return 0;
}

//! ----- ModelItem::insertChildren --------------------------------------------
bool ModelItem::insertChildren(int position, int count, int columns)
{
    if (position < 0 || position > _childItems.size())
        return false;

    for (int row = 0; row < count; ++row) {
        QVector<QVariant> data(columns);
        ModelItem *item = new ModelItem(data, this);
        _childItems.insert(position, item);
    }

    return true;
}


//! ----- ModelItem::removeChildren --------------------------------------------
bool ModelItem::removeChildren(int position, int count)
{
    if (position < 0 || position + count > _childItems.size())
        return false;

    for (int row = 0; row < count; ++row)
        delete _childItems.takeAt(position);

    return true;
}
