/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _MODEL_ITEM_H_
#define _MODEL_ITEM_H_

#include <QStandardItem>
#include <QVariant>


//! ----- Model Item -----------------------------------------------------------
class ModelItem: public QStandardItem
{
 public:
     ModelItem(const QVector<QVariant> &data, ModelItem *parent = 0);
     ~ModelItem();
     
     //! data method
     bool setData(int column, const QVariant &value);
     void setData(const QVector<QVariant> &data);
     QVariant getData(int column) const;

     //! child method
     ModelItem *child(int index);
     int childCount() const;
     int columnCount() const;
     bool insertChildren(int position, int count, int columns);
     int childNumber() const;
     bool removeChildren(int position, int count);
     
     //! parent method
     ModelItem *parent();

     //! playing status method
     void setIsPlaying(bool playing) {_isPlaying = playing;}
     bool isPlaying() {return _isPlaying;}
     
     //! favorite status method
     void setIsFavorite(bool favorite) {_isFavorite = favorite;}
     bool isFavorite() {return _isFavorite;}
    
  private:
     QList<ModelItem*>            _childItems;
     ModelItem                    *_parentItem;
     QVector<QVariant>            _itemData; 
     
     bool                         _isPlaying;
     bool                         _isFavorite;
};

#endif // _MODEL_ITEM_H_
