/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionModel/plsfilepopulator.h"
#include "CollectionModel/plsfilemodel.h"
#include "Database/databasemanager.h"

#include <QFileInfo>
#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>
#include <QVariant>
#include <QDebug>

PlsFilePopulator::PlsFilePopulator()
{
  _plsFileModel    = PlsFileModel::instance();
  _player          = Player::instance();
  _isRunning       = false;
}

void PlsFilePopulator::run()
{
    _isRunning = true;
    qDebug() << " --- PlsFilePopulator --> Starting model population";
    DatabaseManager dbManager("plsfilepopulator");
    if (!dbManager.connectToDB()) {
      _isRunning = false;
      return;
    }
    
    _plsFileModel->clear();

    QModelIndex tracksParent;
    {
        QSqlQuery playlistsQuery("SELECT id,filename,name,type,mtime,favorite FROM playlists ORDER BY type ASC",*dbManager.sqlDb());

        while (playlistsQuery.next()) 
        {
            QVector<QVariant> playlistdata ;
            playlistdata.append( playlistsQuery.value(0).toInt() );      //! 1-playlist id
            playlistdata.append( playlistsQuery.value(1).toString() );   //! 2-playlist filename
            playlistdata.append( playlistsQuery.value(2).toString() );   //! 3-playlist name
            playlistdata.append( playlistsQuery.value(4).toUInt() );     //! 4-date fichier
            playlistdata.append( playlistsQuery.value(3).toInt() );      //! 5-type : 0 interne, 1 user collection

            tracksParent = _plsFileModel->addChild(QModelIndex(),playlistdata);
            bool isPlaylistFavorite = playlistsQuery.value(5).toInt();
            if(isPlaylistFavorite)
                _plsFileModel->setItemIsFavorite(tracksParent, isPlaylistFavorite);       
		  
            QSqlQuery playlistItemQuery("SELECT id,urlname,name FROM playlistitem WHERE playlist="+QString::number(playlistsQuery.value(0).toInt())+"", *dbManager.sqlDb());
		  
            while (playlistItemQuery.next()) 
            {
                QVector<QVariant> itemdata;
                itemdata.append( playlistItemQuery.value(0).toInt());     //! 1-item id
                itemdata.append( playlistItemQuery.value(2).toString());  //! 2-item name
                itemdata.append( playlistItemQuery.value(1).toString());  //! 3-item url
                itemdata.append( QString());                              
  
                _plsFileModel->addChild(tracksParent,itemdata);
            } //! Fin while playlist item
        } //! Fin while playlist
    }
	    
	    
    //! Update of playing item status with current track playing from Player 
    QString currentTrack = _player->currentMediaSource();
    if(!currentTrack.isEmpty()) {
          _plsFileModel->updatePlayingItem(currentTrack);
    }
    qDebug() << " --- PlsFilePopulator --> End model population";

    _isRunning = false;
    emit modelPopulated();
}
