/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _COLLECTION_SCENE_H_
#define _COLLECTION_SCENE_H_

#include "CollectionModel/collectionmodel.h"
#include "CollectionModel/plsfilemodel.h"
#include "CollectionView/graphicitem.h"
#include "CollectionView/virtualplaylist.h"
#include "Database/databasemanager.h"
#include "Player/player.h"

#include <QStringList>
#include <QGraphicsScene>
#include <QtGui>

namespace Collection {
  enum CollectionMode {
      ViewAlbum             = 1,
      ViewArtist            = 2,
      ViewTrack             = 3,
      ViewGenre             = 4,
      ViewFavorite          = 5,
      ViewPlaylist          = 6,
      ViewPlaylistDetail    = 7
  };
}


//!----------------------------------------------------------------------------
//! Collection Scene
//!----------------------------------------------------------------------------
class CollectionScene : public QGraphicsScene
{
Q_OBJECT
  public: 
    CollectionScene(QWidget *parent);
    ~CollectionScene();
    void  setVirtualPlaylist(VirtualPlaylist  *playlist);
    int   getSceneInfoSize() {return _infoSize;}

    Collection::CollectionMode   _collectionMode;
    CollectionModel              *_collectionModel;
    PlsFileModel                 *_plsFileModel;
    QList<CategorieGraphicItem*> _listCategorie;    

  private :
    VirtualPlaylist             *_virtualPlaylist;
    Player                      *_player;
        
    QList<TrackGraphicItem*>    _listTrack;    
    int                         _infoSize;
    QWidget                     *_parentWidget;

    QMenu                       *_menuAlbum;
    QMenu                       *_menuAlbumGenre;
    QMenu                       *_menuArtist;
    QMenu                       *_menuPlaylist;    
    QAction                     *albumFavoriteAct;
    QAction                     *artistFavoriteAct;
    QAction                     *playlistFavoriteAct;
    QGraphicsItem               *_mouseGrabbedItem;
    
  protected:
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);

  private slots :
    //! population scene method
    void populateAlbumScene();
    void populateArtistScene();
    void populateTrackScene();
    void populateGenreScene();
    void populateFavoriteScene();
    void populatePlaylistScene();
    void populatePlaylistDetailScene();
    //! playing item scene method
    void playAlbum();
    void playAlbumGenre();
    void playArtist();
    void playTrack();
    void playPlaylist();
    //! album graphic item method
    void loadCoverArtFromFile();
    void loadLastFmCover();
    void removeCoverArt();
    void modifyAlbumInfo();
    void modifyGenreInfo();
    void modifyArtistInfo();
    void updateAlbumFavorite();
    void updateArtistFavorite();
    void removePlaylistFromDisk();
    void modifyPlaylistInfo();
    void updatePlaylistFavorite();    
    //! update scene method
    void updateMouseGrabbedItem();
    void itemDoubleClick();
    
  public slots:
    void populateScene();
    void updateTrackPlaying(QString filename);

  signals:
    void signal_RepopulateCollection();
    void signal_RepopulatePlsFile();
};

#endif // _COLLECTION_SCENE_H_
