/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "CollectionView/graphicitem.h"

#include "CollectionModel/modelitem.h"
#include "CollectionModel/collectionmodel.h"
#include "CollectionModel/plsfilemodel.h"
#include "constants.h"

#include "RadioModel/streamitem.h"
#include "RadioModel/streammodel.h"
#include <QPen>

namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}



//!-----------------------------------------------------------------------------
//! CategorieGraphicItem 
//!-----------------------------------------------------------------------------
CategorieGraphicItem::CategorieGraphicItem() 
{
   setAcceptsHoverEvents(false);
   setAcceptDrops(false);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
}

QRectF CategorieGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 630, 50);
}

void CategorieGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)

   //! draw categorie text
   painter->setFont(QFont("Trebuchet MS", 10, QFont::Black));
   painter->setPen(QColor(0x315996));
   painter->drawStaticText(5,2, QStaticText(_catName));

   //! draw decoration line
   QColor outlineColor = QColor(0x315996);
   outlineColor.setAlphaF(0.15);
   QColor inlineColor = QColor(0x315996);
   inlineColor.setAlphaF(1);
   
    //BEGIN: horizontal line
    {
        QPoint start(boundingRect().toRect().left()+3, boundingRect().toRect().top()+18);
        start.rx() += 3;
        QPoint horizontalGradTop(boundingRect().toRect().topLeft());
        horizontalGradTop.rx() += boundingRect().toRect().width() - 6;
        QLinearGradient gradient(start, horizontalGradTop);
        gradient.setColorAt(0, inlineColor);
        gradient.setColorAt(1, outlineColor);	
        painter->fillRect(QRect(start, QSize(boundingRect().toRect().width() - 6, 1)), gradient);
    }
    //END: horizontal line
}

//!-----------------------------------------------------------------------------
//! AlbumGraphicItem
//!-----------------------------------------------------------------------------
AlbumGraphicItem::AlbumGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(true);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   setCursor(Qt::OpenHandCursor);
}

QRectF AlbumGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 140, 150);
}

void AlbumGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)
   //! get Model data 
   ModelItem *item = (CollectionModel::instance())->getItem(_modelIdx);
   QString _title     =  item->getData(1).toString();
   QString _year      =  item->getData(2).toString();
   QString _coverPath =  item->getData(3).toString();
   
   QPixmap pixTemp(QSize(140,150));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
 
      //! paint cover art
      QPixmap pix = QPixmap(_CONFIGDIR + "/albums/" + _coverPath);
      if(pix.isNull()) pix = QPixmap(":/images/emptycoverblue110x110.png");
      p.drawPixmap(15,10, pix);
      
      p.setPen(QColor(0x315996));
      p.setFont(QFont("Trebuchet MS", 8, QFont::Normal));
      
      QFontMetrics metric( p.font());
      QString _titleElided = metric.elidedText ( _title, Qt::ElideRight, 280);
      QRect textBox = QRect (0,110,140, 40);
      QRect fontrect = metric.boundingRect (textBox,  Qt::AlignCenter | Qt::TextWordWrap, _titleElided, 0, 0 );
      p.drawText(textBox.adjusted(0, metric.height(), 0, 0) , Qt::AlignCenter | Qt::TextWordWrap, _titleElided);
   
      //! paint year
      p.drawText(QRect(0, 0, 140, 10), Qt::AlignTop | Qt::AlignCenter, _year);      

      //! paint playing or favorite attibute
      if(item->isPlaying())
        p.drawPixmap(118, 0, QPixmap(":/images/media-playing.png"));
      
      if(item->isFavorite())
        p.drawPixmap(118, 30, QPixmap(":/images/media-favorite.png"));

      //! paint opacity if over            
      if (option->state.testFlag(QStyle::State_MouseOver))
        setOpacity(0.5);
      else 
        setOpacity(1);

      p.end();
   }
   painter->drawPixmap(0, 0, pixTemp);
}

void AlbumGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  update();
}

void AlbumGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  update();
}

void AlbumGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
  Q_UNUSED(event)
  //qDebug() << " ---- AlbumGraphicItem::mousePressEvent " ;
}

void AlbumGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
  Q_UNUSED(event) 
}

void AlbumGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event ) 
{
  Q_UNUSED(event)
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value("itemDoubleClick"))->setData(v); 
  (actions->value("itemDoubleClick"))->trigger();
} 



void AlbumGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
  QMimeData *mimedata=new QMimeData;
  QList<QUrl> listUrl;
  
  QStringList _list = (CollectionModel::instance())->getItemChildrenTracks(_modelIdx);

  foreach (const QString &file, _list) {
    listUrl << QUrl(file);
  }
  
  mimedata->setUrls(listUrl);
  QDrag *drag = new QDrag(event->widget());
  drag->setMimeData(mimedata);
  drag->start(); 
}



//!----------------------------------------------------------------------------
//! AlbumGenreGraphicItem
//!----------------------------------------------------------------------------
AlbumGenreGraphicItem::AlbumGenreGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(true);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   setCursor(Qt::OpenHandCursor);
}

QRectF AlbumGenreGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 140, 150);
}

void AlbumGenreGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)
   //! get Model data 
   ModelItem *item = (CollectionModel::instance())->getItem(_modelIdx);
   QString _title     =  item->getData(1).toString();
   QString _year      =  item->getData(2).toString();
   QString _coverPath =  item->getData(3).toString();
   
   QPixmap pixTemp(QSize(140,150));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
 
      //! paint cover art
      QPixmap pix = QPixmap(_CONFIGDIR + "/albums/" + _coverPath);
      if(pix.isNull()) pix = QPixmap(":/images/emptycoverblue110x110.png");
      p.drawPixmap(15,10, pix);
      
      p.setPen(QColor(0x315996));
      p.setFont(QFont("Trebuchet MS", 8, QFont::Normal));
      
      QFontMetrics metric( p.font());
      QString _titleElided = metric.elidedText ( _title, Qt::ElideRight, 280);
      QRect textBox = QRect (0,110,140, 40);
      QRect fontrect = metric.boundingRect (textBox,  Qt::AlignCenter | Qt::TextWordWrap, _titleElided, 0, 0 );
      p.drawText(textBox.adjusted(0, metric.height(), 0, 0) , Qt::AlignCenter | Qt::TextWordWrap, _titleElided);
   
      //! paint year
      p.drawText(QRect(0, 0, 140, 10), Qt::AlignTop | Qt::AlignCenter, _year);      

      //! paint playing or favorite attibute
      if(item->isPlaying() &&  ((CollectionModel::instance())->getPlayingTrackItemGenre() == _genre))
        p.drawPixmap(118, 0, QPixmap(":/images/media-playing.png"));
      if(item->isFavorite())
        p.drawPixmap(118, 30, QPixmap(":/images/media-favorite.png"));

      //! paint opacity if over            
      if (option->state.testFlag(QStyle::State_MouseOver))
        setOpacity(0.5);
      else 
        setOpacity(1);

      p.end();
   }
   painter->drawPixmap(0, 0, pixTemp);
}



void AlbumGenreGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event ) 
{
  Q_UNUSED(event)
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value("itemDoubleClick"))->setData(v); 
  (actions->value("itemDoubleClick"))->trigger();  
} 

void AlbumGenreGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  update();  
}

void AlbumGenreGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  update();  
}

void AlbumGenreGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) 
{
  Q_UNUSED(event)
}

void AlbumGenreGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
  Q_UNUSED(event)
}

void AlbumGenreGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
  QMimeData *mimedata=new QMimeData;
  QList<QUrl> listUrl;
  
  QStringList _list = (CollectionModel::instance())->getAlbumChildrenTracksGenre(_modelIdx,_genre);

  foreach (const QString &file, _list) {
    listUrl << QUrl(file);
  }
  
  mimedata->setUrls(listUrl);
  QDrag *drag = new QDrag(event->widget());
  drag->setMimeData(mimedata);
  drag->start(); 
}

  
//!-----------------------------------------------------------------------------
//! ArtistGraphicItem 
//!-----------------------------------------------------------------------------
ArtistGraphicItem::ArtistGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(true);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   setCursor(Qt::OpenHandCursor);  
}

QRectF ArtistGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 140, 150);
}

void ArtistGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)
   //! get Model data 
   ModelItem *item = (CollectionModel::instance())->getItem(_modelIdx);
   QString      _name          = item->getData(1).toString();
   QStringList  _coverPathList = item->getData(2).toStringList();
   
   QPixmap basePixmap = QPixmap( 140, 150 );
   {
     basePixmap.fill( Qt::transparent );
     QPainter pt( &basePixmap );
     
     //!WARNING limite de l'affichage à 5 cover
     int i = _coverPathList.size() >=5 ? _coverPathList.size() - 5 : 0;
     for (;i <_coverPathList.size(); i++) {
     //for (i; i <_coverPathList.size(); i++) {
         QPixmap pix = QPixmap(_CONFIGDIR + "/albums/" + _coverPathList[i]);
         if(pix.isNull()) pix = QPixmap(":/images/emptycoverblue110x110.png");
         
         pix = pix.scaled(QSize(85,85), Qt::KeepAspectRatio, Qt::SmoothTransformation);
         qreal angle =  15*i;

         QTransform transform = QTransform().rotate(angle, Qt::ZAxis).translate(pix.width()/2,0);
         pix = pix.transformed(transform, Qt::SmoothTransformation); 
      
         pt.drawPixmap(boundingRect().toRect().adjusted((140 -pix.width())/2,
							   (150 -pix.height())/2 -15,
							   -(140 -pix.width())/2, 
							   -(150-pix.height())/2 -15)
							   , pix);
     }
    
     //! affichage nom de l'artist
     pt.setPen(QColor(0x315996));
     pt.setFont(QFont("Trebuchet MS", 8, QFont::Normal));
    
     QRect textBox = boundingRect().toRect().adjusted(0, 100, 0, 0);
     QFontMetrics fontmetric(pt.font());
     QRect fontrect = fontmetric.boundingRect (textBox,  Qt::AlignCenter | Qt::TextWordWrap, _name, 0, 0 );

     pt.drawText(textBox.adjusted(0, fontrect.height(), 0, 0), Qt::AlignCenter | Qt::TextWordWrap, _name);

     //! affichage des attributs playing ou favorite
     if(item->isPlaying())
        pt.drawPixmap(118, 0, QPixmap(":/images/media-playing.png"));
      
     if(item->isFavorite())
        pt.drawPixmap(118, 30, QPixmap(":/images/media-favorite.png"));

      //! paint opacity if over            
      if (option->state.testFlag(QStyle::State_MouseOver))
        setOpacity(0.5);
      else 
        setOpacity(1);
          
     pt.end();
  }
  painter->drawPixmap(boundingRect().toRect(), basePixmap);
}


void ArtistGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event ) 
{
  Q_UNUSED(event)
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value("itemDoubleClick"))->setData(v); 
  (actions->value("itemDoubleClick"))->trigger();  
} 

void ArtistGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  update();  
}

void ArtistGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  update();  
}

void ArtistGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
  Q_UNUSED(event)
}


void ArtistGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
  Q_UNUSED(event)
}

void ArtistGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
  QMimeData *mimedata=new QMimeData;
  QList<QUrl> listUrl;
  
  QStringList _list = (CollectionModel::instance())->getItemChildrenTracks(_modelIdx);

  foreach (const QString &file, _list) {
    listUrl << QUrl(file);
  }
  
  mimedata->setUrls(listUrl);
  QDrag *drag = new QDrag(event->widget());
  drag->setMimeData(mimedata);
  drag->start(); 
}

//!-----------------------------------------------------------------------------
//! TrackGraphicWidget
//!-----------------------------------------------------------------------------
TrackGraphicItem::TrackGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(true);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);  
   setCursor(Qt::OpenHandCursor);   
}

QRectF TrackGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 450, 22);
}

void TrackGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)
   //! get Model data 
   ModelItem *item = (CollectionModel::instance())->getItem(_modelIdx);
   QString _tracknumber =  item->getData(3).toString();
   QString _trackname =  item->getData(1).toString();

  QPixmap pixTemp(QSize(450,22));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
      
      //! paint track number & title
      p.setPen(QColor(0x315996));
      p.setFont(QFont("Trebuchet MS", 8, QFont::Normal));

      p.drawText(QRect(30, 0, 30, 22), Qt::AlignLeft | Qt::AlignVCenter, _tracknumber);
      QFontMetrics metric( p.font());
      QString _tracknameElided = metric.elidedText ( _trackname, Qt::ElideRight, 390);
      p.drawText(QRect(60, 0, 390, 22), Qt::AlignLeft | Qt::AlignVCenter,_tracknameElided);
	
      //! paint activated item
      if(item->isPlaying())
	  p.drawPixmap(0, 0,QPixmap(":/images/media-playing.png"));
      
      p.end();
   }
   painter->drawPixmap(0, 0, pixTemp);   
}

void TrackGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event ) 
{
  Q_UNUSED(event)
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value("itemDoubleClick"))->setData(v); 
  (actions->value("itemDoubleClick"))->trigger();  
} 


void TrackGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
}

void TrackGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
}

void TrackGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
  Q_UNUSED(event)
}


void TrackGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
  Q_UNUSED(event)
}

void TrackGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
  QMimeData *mimedata=new QMimeData;
  QList<QUrl> listUrl;
  
  QStringList _list = (CollectionModel::instance())->getItemChildrenTracks(_modelIdx);

  foreach (const QString &file, _list) {
    listUrl << QUrl(file);
  }
  
  mimedata->setUrls(listUrl);
  QDrag *drag = new QDrag(event->widget());
  drag->setMimeData(mimedata);
  drag->start();
}


//!-----------------------------------------------------------------------------
//! PlaylistGraphicItem
//!-----------------------------------------------------------------------------
PlaylistGraphicItem::PlaylistGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(true);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);
   setCursor(Qt::OpenHandCursor);
}

QRectF PlaylistGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 170, 90);
}


void PlaylistGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)
   //! get Model data 
   ModelItem *item = (PlsFileModel::instance())->getItem(_modelIdx);
   QString _name     =  item->getData(2).toString();
   uint dateInt      =  item->getData(3).toUInt();
   QDateTime date    = QDateTime::fromTime_t(dateInt);
   QString _playlistDate = date.toString("dd.MM.yyyy");
   
   
   QPixmap pixTemp(QSize(170,90));
   {
      pixTemp.fill(Qt::transparent);
      QPainter p;
      p.begin(&pixTemp);
     
      //! paint cover art
      QPixmap pix;
      if(item->getData(4).toInt() == 0)
        pix = QPixmap(":/images/playlist-file-internal-48x48.png");
      else 
        pix = QPixmap(":/images/playlist-file-48x48.png");
      
      p.drawPixmap(boundingRect().toRect().adjusted(61, 10, -61, -112), pix);
      
      //! paint playlist date
      p.setPen(QColor(0x315996));
      p.setFont(QFont("Trebuchet MS", 8, QFont::Normal));
      p.drawText(boundingRect().toRect().adjusted(0, -10, 0, -70), Qt::AlignCenter, _playlistDate);
   
      //! paint playlist name
      QRect textBox = boundingRect().toRect().adjusted(0, 30, 0, 0);
      QFontMetrics fontmetric(p.font());
      QRect fontrect = fontmetric.boundingRect (textBox,  Qt::AlignCenter | Qt::TextWordWrap, _name, 0, 0 );
      
      p.drawText(textBox.adjusted(0, fontrect.height(), 0, 0), Qt::AlignCenter | Qt::TextWordWrap, _name);

      //! paint activated item
      if(item->isPlaying())
        p.drawPixmap(138, 0, QPixmap(":/images/media-playing.png"));

      //! paint favorites item
      if(item->isFavorite()) 
        p.drawPixmap(140, 30, QPixmap(":/images/media-favorite.png"));

      //! paint mouse hover item
      if (option->state.testFlag(QStyle::State_MouseOver))
        setOpacity(0.5);
      else
        setOpacity(1.0);
    
      p.end();
   }
   painter->drawPixmap(0, 0, pixTemp);  
}


void PlaylistGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent *event ) 
{
  Q_UNUSED(event)
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value("itemDoubleClick"))->setData(v); 
  (actions->value("itemDoubleClick"))->trigger();  
} 


void PlaylistGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  update();  
}

void PlaylistGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
  update();  
}

void PlaylistGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
  Q_UNUSED(event)
}

void PlaylistGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
  Q_UNUSED(event)
}

void PlaylistGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
  QMimeData *mimedata=new QMimeData;
  ModelItem *item = (PlsFileModel::instance())->getItem(_modelIdx);
  QList<QUrl> listUrl;
  listUrl.append(QUrl(item->getData(1).toString()));

     
  mimedata->setUrls(listUrl);
  QDrag *drag = new QDrag(event->widget());
  drag->setMimeData(mimedata);
  drag->start();
}

//!-----------------------------------------------------------------------------
//! StreamGraphicItem
//!-----------------------------------------------------------------------------
StreamGraphicItem::StreamGraphicItem()
{
   setAcceptsHoverEvents(true);
   setAcceptDrops(true);
   setFlag(QGraphicsItem::ItemIsSelectable, false);
   setFlag(QGraphicsItem::ItemIsMovable, false);
   setFlag(QGraphicsItem::ItemIgnoresTransformations, true);  
   setCursor(Qt::OpenHandCursor);   
}

QRectF StreamGraphicItem::boundingRect() const
{
    return QRectF(0, 0, 500, 22);
}


void StreamGraphicItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * )
{
Q_UNUSED(option)
   //! get Model data 
   StreamItem *item = (StreamModel::instance())->getStreamItem(_StreamRow);
   QString _streamname =  item->name;
   
   //! paint track title
   painter->setPen(QColor(0x315996));
   painter->setFont(QFont("Trebuchet MS", 8, QFont::Normal));
   painter->drawText(boundingRect().toRect().adjusted(60, 0, -60, 0), Qt::AlignLeft | Qt::AlignVCenter, _streamname);
    
   //! paint activated item
   if(item->isPlaying)
      painter->drawPixmap(0, 0,QPixmap(":/images/media-playing.png"));
}

void StreamGraphicItem::mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event ) 
{
  Q_UNUSED(event)
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
  QVariant v; 
  v.setValue(static_cast<QGraphicsItem*>(this));
  
  (actions->value("streamDoubleClick"))->setData(v); 
  (actions->value("streamDoubleClick"))->trigger();  
} 


void StreamGraphicItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
}

void StreamGraphicItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
  Q_UNUSED(event)
}

void StreamGraphicItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) 
{
  Q_UNUSED(event)
}

void StreamGraphicItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
  Q_UNUSED(event)
}


void StreamGraphicItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
  QMimeData *mimedata=new QMimeData;
  
  StreamItem *item = (StreamModel::instance())->getStreamItem(_StreamRow);
  
  QList<QUrl> listUrl;
  listUrl.append(item->url);
  //qDebug() << " ---- StreamGraphicItem -> listUrl.append(item->url);" << item->url;

  mimedata->setUrls(listUrl);
  QDrag *drag = new QDrag(event->widget());
  drag->setMimeData(mimedata);
  drag->start();
}
