/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _GRAPHICS_ITEM_H_
#define _GRAPHICS_ITEM_H_

#include <QtGui>

namespace GraphicsItem {
  enum GraphicsType {
      AlbumType             = QGraphicsItem::UserType + 1,
      ArtistType            = QGraphicsItem::UserType + 2,
      AlbumGenreType        = QGraphicsItem::UserType + 3,
      TrackType             = QGraphicsItem::UserType + 4,
      PlaylistType          = QGraphicsItem::UserType + 5,
      StreamType            = QGraphicsItem::UserType + 6
  };
}
//!-----------------------------------------------------------------------------
//! CategorieGraphicItem 
//!-----------------------------------------------------------------------------
class CategorieGraphicItem : public QGraphicsItem
{
  public:
    CategorieGraphicItem();
    QString     _catName;

  protected:
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);
};

//!-----------------------------------------------------------------------------
//! AlbumGraphicItem 
//!-----------------------------------------------------------------------------
class AlbumGraphicItem : public QGraphicsItem
{
  public:
    AlbumGraphicItem();
    QModelIndex             _modelIdx;
    int type() const { return GraphicsItem::AlbumType; }

  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
};

//!-----------------------------------------------------------------------------
//! AlbumGenreGraphicItem 
//!-----------------------------------------------------------------------------
class AlbumGenreGraphicItem : public QGraphicsItem
{
  public:
    AlbumGenreGraphicItem();
    QModelIndex             _modelIdx;
    QString                 _genre;
    int type() const { return GraphicsItem::AlbumGenreType; }
    
  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
};

//!-----------------------------------------------------------------------------
//! ArtistGraphicItem 
//!-----------------------------------------------------------------------------
class ArtistGraphicItem : public QGraphicsItem
{
  public:
    ArtistGraphicItem();
    QModelIndex             _modelIdx;
    int type() const { return GraphicsItem::ArtistType; }
    
  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
};

//!-----------------------------------------------------------------------------
//! TrackGraphicWidget 
//!-----------------------------------------------------------------------------
class TrackGraphicItem : public QGraphicsItem
{
  public:
    TrackGraphicItem();
    QModelIndex             _modelIdx;
    int type() const { return GraphicsItem::TrackType; }
    
  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent *event );
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
};


//!----------------------------------------------------------------------------
//! PlaylistGraphicItem 
//!----------------------------------------------------------------------------
class PlaylistGraphicItem : public QGraphicsItem
{
  public:
    PlaylistGraphicItem();
    QModelIndex             _modelIdx;
    int type() const { return GraphicsItem::PlaylistType; }
        
  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
};

//!----------------------------------------------------------------------------
//! StreamGraphicItem 
//!----------------------------------------------------------------------------
class StreamGraphicItem : public QGraphicsItem
{
  public:
    StreamGraphicItem();
    int type() const { return GraphicsItem::StreamType; }
    int              _StreamRow;
    bool             isFavorite;

  protected:
    // Implement QGraphicsItem method
    QRectF boundingRect() const;
    void paint(QPainter *p, const QStyleOptionGraphicsItem *, QWidget *);

    void hoverEnterEvent(QGraphicsSceneHoverEvent *e);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *e);
    void mousePressEvent(QGraphicsSceneMouseEvent *e);
    void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );
    void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
};

#endif // _GRAPHICS_ITEM_H_
