/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionView/userchangedialog.h"
#include "Database/databasemanager.h"
#include "MediaItem/mediaitem.h"
#include "constants.h"

#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>
#include <QVariant>
#include <QString>

//! -------------- AlbumChangeDlg ----------------------------------------------
AlbumChangeDialog::AlbumChangeDialog(int id)
{
  _albumBDId = id;
  _albuminfochanged = false;
  
  //! create Gui
  CreateGui();
  
  //! Connection
  QObject::connect(buttonBox, SIGNAL(accepted()), this, SLOT(on_buttonBox_accepted()));
  QObject::connect(buttonBox, SIGNAL(rejected()), this, SLOT(on_buttonBox_rejected()));


  //! Get current album Info
  DatabaseManager dbManager("AlbumItem");
  if (dbManager.connectToDB()) {
    QSqlQuery tracksQuery("SELECT filename,album,interpret,year FROM view_tracks WHERE albumID="+QString::number(_albumBDId)+" ORDER BY track ASC;",
				  *dbManager.sqlDb());

    tracksQuery.next();
    if (tracksQuery.isValid()) {
      _album  = tracksQuery.value(1).toString();
      _artist = tracksQuery.value(2).toString();
      _year   = tracksQuery.value(3).toString();
      lineEdit->setText(_year);   
      lineEdit_2->setText(_artist); 
      lineEdit_3->setText(_album); 
    }
  }
  else
  {
	  lineEdit->setText("Connection to Database failed");
	  lineEdit_2->setText("Connection to Database failed");
	  lineEdit_3->setText("Connection to Database failed");
	  buttonBox->setStandardButtons(QDialogButtonBox::Cancel);
  }
}


AlbumChangeDialog::~AlbumChangeDialog()
{
}


void AlbumChangeDialog::CreateGui()
{
    this->resize(445, 157);
    this->setWindowTitle(QString::fromUtf8("YaRock - Change album Info"));

    QWidget     *gridLayoutWidget;
    QGridLayout *gridLayout;
    QLabel      *label, *label_2, *label_3;
 
    buttonBox = new QDialogButtonBox(this);
    buttonBox->setGeometry(QRect(140, 110, 291, 32));
    buttonBox->setOrientation(Qt::Horizontal);
    buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
    
    gridLayoutWidget = new QWidget(this);
    gridLayoutWidget->setGeometry(QRect(10, 10, 431, 101));
        
    gridLayout = new QGridLayout(gridLayoutWidget);
    gridLayout->setContentsMargins(0, 0, 0, 0);

    label = new QLabel("Album name", gridLayoutWidget);
    gridLayout->addWidget(label, 0, 0, 1, 1);

    label_2 = new QLabel("Artist name", gridLayoutWidget);
    gridLayout->addWidget(label_2, 1, 0, 1, 1);

    label_3 = new QLabel("Year", gridLayoutWidget);
    gridLayout->addWidget(label_3, 2, 0, 1, 1);

    lineEdit = new QLineEdit(gridLayoutWidget);
    gridLayout->addWidget(lineEdit, 2, 1, 1, 1);

    lineEdit_2 = new QLineEdit(gridLayoutWidget);
    gridLayout->addWidget(lineEdit_2, 1, 1, 1, 1);

    lineEdit_3 = new QLineEdit(gridLayoutWidget);
    gridLayout->addWidget(lineEdit_3, 0, 1, 1, 1);
}



void AlbumChangeDialog::on_buttonBox_accepted()
{
  DatabaseManager dbManager("AlbumItem");
  if (!dbManager.connectToDB()) return;
  QStringList _list;
  
  QSqlField data("col",QVariant::String);
  
  QSqlQuery("BEGIN TRANSACTION;",*dbManager.sqlDb());
  
  //! Select track from album
  QSqlQuery tracksQuery("SELECT `filename`,`trackname`,`track`,`length`,`album`,`interpret`,`genre`,`year` FROM `view_tracks` WHERE `albumID`="+QString::number(_albumBDId)+";",
				  *dbManager.sqlDb());

  QString newCoverName;
  QString oldCoverName;  

  while(tracksQuery.next())
  {
    //! Get filename
    QString filename = tracksQuery.value(0).toString();
    data.setValue(filename);
    QString filenameDB = dbManager.sqlDb()->driver()->formatValue(data,false);
    QFileInfo fileInfo(filename);
    
    //! WARNING : in case of year modification, we never quit the while loop FIXME
    if(!_list.contains(filename))
      _list << filename;
    else
      break;

    //! Get tags from DB
    MediaItem *_mediaitem = new MediaItem();
    
    _mediaitem->title         = tracksQuery.value(1).toString();
    _mediaitem->num           = tracksQuery.value(2).toInt();
    _mediaitem->duration      = tracksQuery.value(3).toInt();
    _mediaitem->album         = tracksQuery.value(4).toString();
    _mediaitem->artist        = tracksQuery.value(5).toString();
    _mediaitem->genre         = tracksQuery.value(6).toString();
    
    data.setValue(_mediaitem->genre);
    _mediaitem->genre = dbManager.sqlDb()->driver()->formatValue(data,false);    
    
    _mediaitem->year = tracksQuery.value(7).toInt();
    uint mtime = fileInfo.lastModified().toTime_t();
    
    oldCoverName = MEDIA::getCoverName(_mediaitem);

    //! update tags with user values
    QString s_year;
    if(_artist != lineEdit_2->text()) {
      _mediaitem->artist = lineEdit_2->text();
      _albuminfochanged = true;
    }
    if(_album != lineEdit_3->text()) {
      _mediaitem->album = lineEdit_3->text();
      _albuminfochanged = true;
    }
    if(_year != lineEdit->text()) {
      data.setValue( (lineEdit->text()).toUInt() );
      s_year = dbManager.sqlDb()->driver()->formatValue(data,false);   
      _albuminfochanged = true;
    }
    else {
      data.setValue(_mediaitem->year);
      s_year = dbManager.sqlDb()->driver()->formatValue(data,false);   
    }

    newCoverName = MEDIA::getCoverName(_mediaitem);
    
    data.setValue(_mediaitem->artist);
    _mediaitem->artist = dbManager.sqlDb()->driver()->formatValue(data,false);

    data.setValue(_mediaitem->album);
    _mediaitem->album = dbManager.sqlDb()->driver()->formatValue(data,false);

    //! DATABASE MODIFICATION : remove track from db
    QSqlQuery query("DELETE FROM `tracks` WHERE `filename`="+filenameDB+";", *dbManager.sqlDb());

    //! DATABASE MODIFICATION : YEAR part in database
    {
	    QSqlQuery query("SELECT `id` FROM `years` WHERE `year`="+s_year+";", *dbManager.sqlDb());
	    query.next();
	    if (!query.isValid())
	      QSqlQuery query("INSERT INTO `years`(`year`) VALUES("+s_year+");", *dbManager.sqlDb());
    }
      
    //! DATABASE MODIFICATION : ARTIST part in database
    {
	    QSqlQuery query("SELECT `id` FROM `interprets` WHERE `interpret`="+_mediaitem->artist+";", *dbManager.sqlDb());
	    query.next();
	    if (!query.isValid())
        QSqlQuery query("INSERT INTO `interprets`(`interpret`) " \
			      "VALUES("+_mediaitem->artist+");", *dbManager.sqlDb());
	  }
      
    //! DATABASE MODIFICATION : ALBUM part in database
    data.setValue(newCoverName);
    QString coverName_Db = dbManager.sqlDb()->driver()->formatValue(data,false);

    QString interpretID;
    {
	    QSqlQuery artistquery("SELECT `id` FROM `interprets` WHERE `interpret`="+_mediaitem->artist+";", *dbManager.sqlDb());
	    artistquery.next();
	    data.setValue(artistquery.value(0).toString());
	    interpretID = dbManager.sqlDb()->driver()->formatValue(data,false);
	  
	    QSqlQuery query("SELECT `id`,`album`,`interpretID` FROM `albums` WHERE `album`="+_mediaitem->album+" AND `interpretID`="+interpretID+";", *dbManager.sqlDb());
	    query.next();
	    if (!query.isValid()) {
        qDebug() << " ----new album /ARTIST ident---- " << _mediaitem->album + " " + interpretID;  
      
        QSqlQuery query("INSERT INTO `albums`(`album`, `interpretID`, `cover`, `year`) " \
			      "VALUES("+_mediaitem->album+","+interpretID+", "+coverName_Db+", "+s_year+");", *dbManager.sqlDb());	      
	    }
	    else {
	      data.setValue(query.value(0).toString());
	      QString albumID = dbManager.sqlDb()->driver()->formatValue(data,false);
	      QSqlQuery query("UPDATE `albums` " \
			      "SET  `year`="+s_year+" " \
			      "WHERE `id`="+albumID+";", *dbManager.sqlDb());
			      

	    }
    } 

    //! DATABASE MODIFICATION : TRACK part in database
    {
	    QSqlQuery query(*dbManager.sqlDb());
	    query.prepare("INSERT INTO `tracks`(`filename`,`trackname`,`track`,`length`,`interpret`,`album`,`year`,`genre`,`mtime`)" \
			  "VALUES(?," \
			  "       ?," \
			  "       ?," \
			  "       ?," \
			  "       (SELECT `id` FROM `interprets` WHERE `interpret`="+_mediaitem->artist+")," \
			  "       (SELECT `id` FROM `albums` WHERE `album`="+_mediaitem->album+" AND `interpretID`="+interpretID+")," \
			  "       (SELECT `id` FROM `years` WHERE `year`="+s_year+")," \
			  "       (SELECT `id` FROM `genres` WHERE `genre`="+_mediaitem->genre+")," \
			  "       ?);");
	    query.addBindValue(filename);
	    query.addBindValue(_mediaitem->title);
	    query.addBindValue(_mediaitem->num);
	    query.addBindValue(_mediaitem->duration);
	    query.addBindValue(mtime);
	    query.exec();
    }
  } //! end WHILE Select track from album

  //! Just once , If old cover exist -> take it !!
  recupCoverArt(newCoverName,oldCoverName);
	      
  //! DATABASE MODIFICATION : CLEAN 
  {
  QSqlQuery query("DELETE FROM `albums` WHERE `id` NOT IN (SELECT `album` FROM `tracks` GROUP BY `album`);", *dbManager.sqlDb());
  }
  {
  QSqlQuery query("DELETE FROM `genres` WHERE `id` NOT IN (SELECT `genre` FROM `tracks` GROUP BY `genre`);", *dbManager.sqlDb());
  }
  {      
  QSqlQuery query("DELETE FROM `interprets` WHERE `id` NOT IN (SELECT `interpret` FROM `tracks` GROUP BY `interpret`);", *dbManager.sqlDb());
  }
  {
  QSqlQuery query("DELETE FROM `years` WHERE `id` NOT IN (SELECT `year` FROM `tracks` GROUP BY `year`);", *dbManager.sqlDb());
  }
  
  
  QSqlQuery("COMMIT TRANSACTION;",*dbManager.sqlDb());
  if(_albuminfochanged)
    emit signal_AlbumInfoChanged();
  
  this->close();
}


void AlbumChangeDialog::on_buttonBox_rejected()
{
    this->close();
}


//! -------------- ArtistChangeDialog ------------------------------------------
ArtistChangeDialog::ArtistChangeDialog(int id)
{
  _artistBDId = id;
  _artistinfochanged = false;
  
  //! create Gui
  CreateGui();
  
  //! Connection
  QObject::connect(buttonBox, SIGNAL(accepted()), this, SLOT(on_buttonBox_accepted()));
  QObject::connect(buttonBox, SIGNAL(rejected()), this, SLOT(on_buttonBox_rejected()));


  //! Get current album Info
  DatabaseManager dbManager("ArtistItem");
  if (dbManager.connectToDB()) {
    QSqlQuery artistQuery("SELECT interpret FROM interprets WHERE id="+QString::number(_artistBDId)+";",*dbManager.sqlDb());

    artistQuery.next();
    if (artistQuery.isValid()) {
      _artist = artistQuery.value(0).toString();
      lineEdit->setText(_artist); 
    }
  }
  else
  {
    lineEdit->setText("Connection to Database failed");
    buttonBox->setStandardButtons(QDialogButtonBox::Cancel);
  }
}


ArtistChangeDialog::~ArtistChangeDialog()
{
}


void ArtistChangeDialog::CreateGui()
{
    this->resize(445, 157);
    this->setWindowTitle(QString::fromUtf8("YaRock - Change artist Info"));
    
    QWidget     *gridLayoutWidget;
    QGridLayout *gridLayout;
    QLabel      *label;
 
    buttonBox = new QDialogButtonBox(this);
    buttonBox->setGeometry(QRect(140, 110, 291, 32));
    buttonBox->setOrientation(Qt::Horizontal);
    buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
    
    gridLayoutWidget = new QWidget(this);
    gridLayoutWidget->setGeometry(QRect(10, 10, 431, 101));
        
    gridLayout = new QGridLayout(gridLayoutWidget);
    gridLayout->setContentsMargins(0, 0, 0, 0);

    label = new QLabel("Artist name", gridLayoutWidget);
    gridLayout->addWidget(label, 0, 0, 1, 1);

    lineEdit = new QLineEdit(gridLayoutWidget);
    gridLayout->addWidget(lineEdit, 0, 1, 1, 1);
}


void ArtistChangeDialog::on_buttonBox_accepted()
{
  DatabaseManager dbManager("AlbumItem");
  if (!dbManager.connectToDB()) return;
  
  QSqlField data("col",QVariant::String);
  
  qDebug() << " ----on_buttonBox_accepted ---- ";  

  QSqlQuery("BEGIN TRANSACTION;",*dbManager.sqlDb());
  
  //! ARTIST HAS CHANGED ?
  data.setValue(lineEdit->text());
  QString artistBD = dbManager.sqlDb()->driver()->formatValue(data,false);

  data.setValue(_artistBDId);
  QString artistBDId = dbManager.sqlDb()->driver()->formatValue(data,false);

  if(_artist != lineEdit->text()) {
     qDebug() << " ----on_buttonBox_accepted Artist different---- ";  
      
     //! check if artist already exist
     QSqlQuery artistQuery("SELECT `id` FROM `interprets` WHERE `interpret`="+artistBD+";",*dbManager.sqlDb());
     artistQuery.next();
     if (!artistQuery.isValid()) {
       qDebug() << " ----Artist n'existe pas ---- " << lineEdit->text();  
       QSqlQuery query("INSERT INTO `interprets`(`interpret`) VALUES("+artistBD+");",*dbManager.sqlDb());
     }
     
     
     QSqlQuery artistQuery2("SELECT `id` FROM `interprets` WHERE `interpret`="+artistBD+";",*dbManager.sqlDb());
     artistQuery2.next();
     data.setValue(artistQuery2.value(0).toString());
     QString newartistBDId = dbManager.sqlDb()->driver()->formatValue(data,false);
  
     //! Update Album table
     QSqlQuery albumQuery("SELECT `id`,`album`,`cover` FROM `albums` WHERE `interpretID`="+artistBDId+";",*dbManager.sqlDb());
     while(albumQuery.next()) {
     
       QString oldCoverName = albumQuery.value(2).toString();
       //qDebug() << " ----oldCoverName ---- " <<oldCoverName;  

       QString newCoverName = MEDIA::getCoverName(lineEdit->text(), albumQuery.value(1).toString());
       //qDebug() << " ----newCoverName ---- " <<newCoverName;  

       data.setValue(newCoverName);
       QString coverpathBD = dbManager.sqlDb()->driver()->formatValue(data,false);
       
       QSqlQuery query("UPDATE `albums` "                  \
                     "SET `cover`="+coverpathBD+","        \
                     "    `interpretID`="+newartistBDId+" " \
                     "WHERE `id`="+albumQuery.value(0).toString()+";",*dbManager.sqlDb());
    
       //! If old cover exist -> take it !!
       recupCoverArt(newCoverName, oldCoverName);
       
     } //! WHILE album query
     
    //! Update Tracks table
    QSqlQuery query2("UPDATE `tracks` SET `interpret`="+newartistBDId+" WHERE `interpret`="+artistBDId+" ;",*dbManager.sqlDb());
    
    //! CLEAN 
    QSqlQuery query3("DELETE FROM `interprets` WHERE `id` NOT IN (SELECT `interpret` FROM `tracks` GROUP BY `interpret`);", *dbManager.sqlDb());
    _artistinfochanged = true;
  }

  QSqlQuery("COMMIT TRANSACTION;",*dbManager.sqlDb());
  
  if(_artistinfochanged)
    emit signal_ArtistInfoChanged();
  
  this->close();
}


void ArtistChangeDialog::on_buttonBox_rejected()
{
    this->close();
}


//! -------------- recupCoverArt ------------------------------------------
void recupCoverArt(QString newCoverName, QString oldCoverName)
{
  //qDebug() << " ----recupCoverArt newCoverName + oldCoverName---- " << newCoverName + "   " + oldCoverName;  

  QString storageLocation = _CONFIGDIR + "/albums/";
  
  //! quit if same filename
  if(newCoverName == oldCoverName) return;
   
  //! remove new cover if already exist
  QFile newCoverFile(storageLocation + newCoverName);
  if(newCoverFile.exists()) newCoverFile.remove();

  //! copy old into new
  QFile oldCoverFile(storageLocation + oldCoverName);
  if(oldCoverFile.exists()) { 
    qDebug() << " ----recupCoverArt oldCoverName exists COPY ---- ";  

    oldCoverFile.rename(storageLocation + newCoverName);
  }
}



//! -------------- GenreChangeDialog -------------------------------------------
GenreChangeDialog::GenreChangeDialog(int id, QString genre)
{
  _albumBDId = id;
  _genre = genre;
    
  //! create Gui
  CreateGui();
  
  //! Connection
  QObject::connect(buttonBox, SIGNAL(accepted()), this, SLOT(on_buttonBox_accepted()));
  QObject::connect(buttonBox, SIGNAL(rejected()), this, SLOT(on_buttonBox_rejected()));

 //! Genre is set with Current AlbumGenreItem value
 lineEdit->setText(_genre);   
}


GenreChangeDialog::~GenreChangeDialog()
{
}


void GenreChangeDialog::CreateGui()
{
    this->resize(445, 157);
    this->setWindowTitle(QString::fromUtf8("YaRock - Change album Info"));

    QWidget     *gridLayoutWidget;
    QGridLayout *gridLayout;
    QLabel      *label;
 
    buttonBox = new QDialogButtonBox(this);
    buttonBox->setGeometry(QRect(140, 110, 291, 32));
    buttonBox->setOrientation(Qt::Horizontal);
    buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
    
    gridLayoutWidget = new QWidget(this);
    gridLayoutWidget->setGeometry(QRect(10, 10, 431, 101));
        
    gridLayout = new QGridLayout(gridLayoutWidget);
    gridLayout->setContentsMargins(0, 0, 0, 0);

    label = new QLabel("Genre", gridLayoutWidget);
    gridLayout->addWidget(label, 0, 0, 1, 1);

    lineEdit = new QLineEdit(gridLayoutWidget);
    gridLayout->addWidget(lineEdit, 0, 1, 1, 1);
}

void GenreChangeDialog::on_buttonBox_accepted()
{
  //! check if there is user modification
  QString newGenre = lineEdit->text(); 
  if(_genre == newGenre) {
    this->close();
    return;
  }  

qDebug() << " ----GenreChangeDialog -> genre is different ----";

  DatabaseManager dbManager("AlbumGenreItem");
  if (!dbManager.connectToDB()) return;
  
  QSqlField data("col",QVariant::String);  
  QSqlQuery("BEGIN TRANSACTION;",*dbManager.sqlDb());

  data.setValue(newGenre);
  newGenre = dbManager.sqlDb()->driver()->formatValue(data,false);

  //! DATABASE MODIFICATION : GENRE part in database
  {
	  QSqlQuery query("SELECT `id` FROM `genres` WHERE `genre`="+newGenre+";", *dbManager.sqlDb());
	  
	  qDebug() << " ----GenreChangeDialog -> query.next() ----" << query.next();

	  if (!query.isValid())
	    QSqlQuery query("INSERT INTO `genres`(`genre`) VALUES("+newGenre+");", *dbManager.sqlDb());
  }

  QSqlQuery genreQuery("SELECT `id` FROM `genres` WHERE `genre`="+newGenre+";", *dbManager.sqlDb());
  genreQuery.next();

  data.setValue(genreQuery.value(0).toString());
  QString newGenreID = dbManager.sqlDb()->driver()->formatValue(data,false);

  data.setValue(_genre);
  _genre = dbManager.sqlDb()->driver()->formatValue(data,false);
    
  //! DATABASE MODIFICATION : TRACKS part in database
  QSqlQuery tracksQuery("SELECT `id` FROM `view_tracks` WHERE `albumID`="+QString::number(_albumBDId)+" AND `genre`="+_genre+";",
				  *dbManager.sqlDb());

  while(tracksQuery.next())
  {
    QSqlQuery query("UPDATE `tracks` "       \
                   "SET `genre`="+newGenreID+" " \
                   "WHERE `id`="+tracksQuery.value(0).toString()+";",*dbManager.sqlDb());

    //qDebug() << " ----GenreChangeDialog -> query.exec() ----" << query.exec();

  }
	      
  //! DATABASE MODIFICATION : CLEAN 
  {
  QSqlQuery query("DELETE FROM `genres` WHERE `id` NOT IN (SELECT `genre` FROM `tracks` GROUP BY `genre`);", *dbManager.sqlDb());
  }
  
  QSqlQuery("COMMIT TRANSACTION;",*dbManager.sqlDb());
  emit signal_GenreInfoChanged();
  
  this->close();
}


void GenreChangeDialog::on_buttonBox_rejected()
{
    this->close();
}









//! -------------- PlaylistChangeDialog ------------------------------------------
PlaylistChangeDialog::PlaylistChangeDialog(int id)
{
  _playlistBDId = id;
  _playlistinfochanged = false;
    qDebug() << " ----PlaylistChangeDialog _playlistBDId---- " << _playlistBDId;  

  //! create Gui
  CreateGui();
  
  //! Connection
  QObject::connect(buttonBox, SIGNAL(accepted()), this, SLOT(on_buttonBox_accepted()));
  QObject::connect(buttonBox, SIGNAL(rejected()), this, SLOT(on_buttonBox_rejected()));


  //! Get current album Info
  DatabaseManager dbManager("PLaylistItem");
  if (dbManager.connectToDB()) {
    QSqlQuery playlistQuery("SELECT name FROM playlists WHERE id="+QString::number(_playlistBDId)+";",*dbManager.sqlDb());

    playlistQuery.next();
    if (playlistQuery.isValid()) {
      _playlist = playlistQuery.value(0).toString();
      lineEdit->setText(_playlist); 
    }
  }
  else
  {
    lineEdit->setText("Connection to Database failed");
    buttonBox->setStandardButtons(QDialogButtonBox::Cancel);
  }
}


PlaylistChangeDialog::~PlaylistChangeDialog()
{
}


void PlaylistChangeDialog::CreateGui()
{
    this->resize(445, 157);
    this->setWindowTitle(QString::fromUtf8("YaRock - Change playlist Info"));
    
    QWidget     *gridLayoutWidget;
    QGridLayout *gridLayout;
    QLabel      *label;
 
    buttonBox = new QDialogButtonBox(this);
    buttonBox->setGeometry(QRect(140, 110, 291, 32));
    buttonBox->setOrientation(Qt::Horizontal);
    buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
    
    gridLayoutWidget = new QWidget(this);
    gridLayoutWidget->setGeometry(QRect(10, 10, 431, 101));
        
    gridLayout = new QGridLayout(gridLayoutWidget);
    gridLayout->setContentsMargins(0, 0, 0, 0);

    label = new QLabel("PLaylist name", gridLayoutWidget);
    gridLayout->addWidget(label, 0, 0, 1, 1);

    lineEdit = new QLineEdit(gridLayoutWidget);
    gridLayout->addWidget(lineEdit, 0, 1, 1, 1);
}


void PlaylistChangeDialog::on_buttonBox_accepted()
{
  DatabaseManager dbManager("PLaylistItem");
  if (!dbManager.connectToDB()) return;
  
  QSqlField data("col",QVariant::String);
  
  qDebug() << " ----on_buttonBox_accepted ---- ";  

  QSqlQuery("BEGIN TRANSACTION;",*dbManager.sqlDb());
  
  //! ARTIST HAS CHANGED ?
  data.setValue(lineEdit->text());
  QString playlistNameBD = dbManager.sqlDb()->driver()->formatValue(data,false);

  if(_playlist != lineEdit->text()) {
    
       QSqlQuery query("UPDATE `playlists` "         \
                     "SET `name`="+playlistNameBD+" " \
                     "WHERE `id`="+QString::number(_playlistBDId)+";",*dbManager.sqlDb());
    
   _playlistinfochanged = true;
  }

  QSqlQuery("COMMIT TRANSACTION;",*dbManager.sqlDb());
  
  if(_playlistinfochanged)
    emit signal_PlaylistInfoChanged();
  
  this->close();
}


void PlaylistChangeDialog::on_buttonBox_rejected()
{
    this->close();
}
