/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CollectionView/virtualplaylist.h"
#include <QString>
#include <QStringList>


VirtualPlaylist::VirtualPlaylist(QWidget *parent)
{
Q_UNUSED(parent)
  _PlayingIdx    = -1;
  _isPlaying     = false;
}

void VirtualPlaylist::AppendFileInPlaylist(const QString &file)
{
    if (file.isEmpty()) return;
    _PlayingIdx = 0;
    _listFiles.clear();
    _listFiles.append(file);
    _isPlaying = true;  
    emit signal_collection_playTrack();
}

void VirtualPlaylist::AppendFilesInPlaylist(const QStringList& list)
{
    if (list.isEmpty()) return;
    _PlayingIdx = 0;
    _listFiles.clear();
    _listFiles.append(list);
    _isPlaying = true;
    emit signal_collection_playTrack();
}


void VirtualPlaylist::AppendFilesInPlaylist(const QStringList& list, int startIdx) 
{
    if (list.isEmpty()) return;
    if(startIdx >= 0 && startIdx < list.size()) {
      _PlayingIdx = startIdx;
      _listFiles.clear();
      _listFiles.append(list);
      _isPlaying = true;
      emit signal_collection_playTrack();
    }
}
   
    

void VirtualPlaylist::ClearPlaylist()
{
    _listFiles.clear();
    _PlayingIdx = -1;
    _isPlaying = false;
}

QString VirtualPlaylist::GetNextTrack()
{
  QString file;
       
  if(_PlayingIdx < _listFiles.size()- 1) {
    file = _listFiles[++_PlayingIdx];
  }

  return (file);  
}


QString VirtualPlaylist::GetPrevTrack()
{
  QString file;

  if(_PlayingIdx > 0) {
    file = _listFiles[--_PlayingIdx];
  }

  return (file);  
}


QString VirtualPlaylist::GetActiveTrack()
{
  QString file;
  if( (_PlayingIdx > -1 && _PlayingIdx < _listFiles.size()) ||
      (_PlayingIdx == 0 && _listFiles.size() ==1))
  {
    file = _listFiles[_PlayingIdx];
  }
  return (file); 
  
}

void  VirtualPlaylist::StopPlaying() {
  _PlayingIdx = -1;
  _isPlaying = false;  
}

