/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _DATABASE_PLS_BUILDER_H_
#define _DATABASE_PLS_BUILDER_H_

#include <QRunnable>
#include <QObject>
#include <QStringList>
#include <QSqlDatabase>
#include <QString>
#include <QHash>

//! --------- DataBaseBuilder -------------------------------------------------- 
// Thread thread that : 
//   - parse collection directory for Playlist file
//   - parse internal directory for playlist file
//   - read playlist contents 
//   - write sql database with playlist information
class DataBasePlsBuilder : public QObject, public QRunnable
{
Q_OBJECT
  public:
    DataBasePlsBuilder();
    void run();
    bool isRunning() {return _isRunning;}
      
  public slots:
    void rebuildFolder(QStringList folder);
    void rebuildOnlyInternalFolder();
  
  private:
    QHash<QString,uint>  dbFiles;  
    QStringList          _folders;
    bool                 _isRunning;
        
    void insertPlaylist(QString filename, QSqlDatabase *sqlDb);
    void updatePlaylist(QString filename, QSqlDatabase *sqlDb);
    void removePlaylist(QString filename, QSqlDatabase *sqlDb);
    void cleanUpDatabase(QSqlDatabase *sqlDb);

  signals:
    void buildingFinished();
};

#endif // _DATABASE_PLS_BUILDER_H_
