/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "FileViewer/fileviewer.h"
#include "Database/databasemanager.h"
#include "MediaItem/mediamimedata.h"

#include <QSqlQuery>
#include <QDir>


FileViewer::FileViewer(QWidget *parent) : QTreeView(parent)
{
  Q_UNUSED(parent)
  
  //! file model
  fileSystemModel = new QFileSystemModel();
  fileSystemModel->setReadOnly( true );
  fileSystemModel->setFilter(QDir::AllDirs | QDir::NoDotAndDotDot);
  fileSystemModel->setRootPath(fileSystemModel->rootPath());
  
  //! QTreeView
  this->setDragEnabled(true);
  this->setDragDropMode(QAbstractItemView::DragOnly);
  this->setSelectionMode(QAbstractItemView::SingleSelection);
  this->setVerticalScrollMode(QAbstractItemView::ScrollPerItem);
  this->setAutoExpandDelay(5);
  this->setUniformRowHeights(true);
  this->setSortingEnabled(true);
  this->setAnimated(false);
  this->setHeaderHidden(true);

  this->setModel(fileSystemModel);
  this->hideColumn(1);
  this->hideColumn(2);
  this->hideColumn(3);
  this->setAllColumnsShowFocus(true);
  this->sortByColumn(0, Qt::AscendingOrder);
  this->setCurrentIndex(fileSystemModel->index(QDir::homePath()));
  
}



void FileViewer::mousePressEvent( QMouseEvent* event )
{
  QTreeView::mousePressEvent(event);
}


void FileViewer::mouseMoveEvent(QMouseEvent *event) 
{
  Q_UNUSED(event)

  QString filepath = fileSystemModel->filePath(this->currentIndex());
    
  QDir dirlist(filepath);
  QStringList filters = QStringList() << "*.mp3" << "*.ogg" << "*.flac";
  dirlist.setNameFilters(filters);
  
  QDirIterator dirIterator(dirlist,QDirIterator::Subdirectories);


  QList<QUrl> listUrl;
  QMimeData *mimedata=new QMimeData; 
  
  while (dirIterator.hasNext()) {
    dirIterator.next();
    QFileInfo fileInfo = dirIterator.fileInfo();
    if (fileInfo.isFile()) {
      listUrl << QUrl(fileInfo.filePath().toUtf8());
    }
  }
  
  if(!listUrl.isEmpty()) {
    mimedata->setUrls(listUrl);
    QDrag *drag=new QDrag(this);
    drag->setMimeData(mimedata);
    drag->start();     
  }
}

