/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "mediaitem.h"

/*------------------------------------*/
// for FromDataBase
#include "Database/databasemanager.h"
#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlQuery>
#include <QtSql/QSqlDriver>
#include <QtSql/QSqlField>
#include <QVariant>
/*------------------------------------*/
#include <QFileInfo>

#include <QByteArray>
#include <QBuffer>
#include <QFile>
#include <QImage>
#include <QTime>
#include <QPixmap>
#include <phonon/backendcapabilities.h>

#include <taglib/mpegfile.h>
#include <taglib/fileref.h>
#include <taglib/tstring.h>
#include <taglib/id3v2tag.h>
#include <taglib/attachedpictureframe.h>


//! ---------------------- FromLocalFile ---------------------------------------
MediaItem* MEDIA::FromLocalFile(const QString url)
{
    //qDebug() << " ---- FromLocalFile " << url;
    //! URL check 
    if(url.isEmpty()) return 0; 

    MediaItem * mediaItem = new MediaItem();
    mediaItem->url        = QFileInfo(url).absoluteFilePath().toUtf8();
    mediaItem->filename   = QFileInfo(url).baseName();
    
    //! encodedName shall be valid as long as fileName exists
    QByteArray fileName = QFile::encodeName( url );
    const char * encodedName = fileName.constData(); 

    if (!encodedName) {
      qDebug() << " ---- FromLocalFile -> media file access failed" << url;
      mediaItem->isExists   = false;
      return mediaItem; 
    }
        
    //! TagLib reference 
    TagLib::FileRef fileref = TagLib::FileRef( encodedName, true);
    if (fileref.isNull()) {
      qDebug() << " ---- FromLocalFile -> TagLib file access Failed " << url;
      mediaItem->isExists   = false;
      return mediaItem; 
    }
  
    //! Tag reading 
    TagLib::Tag *tag = 0;
    tag = fileref.tag();
    if ( tag ) {
        mediaItem->title      = TStringToQString(tag->title()).trimmed();
        mediaItem->artist     = TStringToQString(tag->artist()).trimmed();
        mediaItem->album      = TStringToQString(tag->album()).trimmed();
        mediaItem->genre      = TStringToQString(tag->genre()).trimmed();
        mediaItem->num        = tag->track();
        mediaItem->year       = tag->year();
    }
    //! Lenght reading
    TagLib::AudioProperties *audioProperties = fileref.audioProperties();
    if (audioProperties)             
        mediaItem->duration   = audioProperties->length();
    else
        mediaItem->duration   = 0;
    
    if ( TagLib::MPEG::File *file = dynamic_cast<TagLib::MPEG::File *>( fileref.file() ) ) {
      if ( file->ID3v2Tag() ) {
          if ( !file->ID3v2Tag()->frameListMap()["TPE2"].isEmpty() )
          	  mediaItem->albumArtist = TStringToQString( file->ID3v2Tag()->frameListMap()["TPE2"].front()->toString() ).trimmed();
	
	        if(!mediaItem->albumArtist.isEmpty())
            mediaItem->artist = mediaItem->albumArtist;
      }
    }

    //!DEFAULT values if empty tag 
    if (mediaItem->artist.isEmpty()) 
      mediaItem->artist = "unknown artist";
    
    if (mediaItem->album.isEmpty()) 
      mediaItem->album = "unknown album";

    if (mediaItem->genre.isEmpty()) 
      mediaItem->genre = "unknown genre";
  
    if (mediaItem->title.isEmpty())
      mediaItem->title = mediaItem->filename;
      
    return mediaItem;
}

//! ---------------------- FromDataBase ----------------------------------------
MediaItem* MEDIA::FromDataBase(const QString url)
{
  MediaItem * _mediaItem = new MediaItem();
  
  //! search track info into database
  DatabaseManager dbManager("MediaItem From DataBase");

  //! Try database connection
  if (dbManager.connectToDB()) {
    //qDebug() << " Media Item from db : " << url;
    QSqlField data("col",QVariant::String);
    data.setValue(QFileInfo(url).absoluteFilePath());
    QString fname = dbManager.sqlDb()->driver()->formatValue(data,false);
    
    QSqlQuery tracksQuery("SELECT id,filename,trackname,track,length,interpret,genre,album,year FROM view_tracks WHERE filename="+fname+" LIMIT 1;",
                                              *dbManager.sqlDb());
    
    if (tracksQuery.first()) {
      //qDebug() << " Build MediaItem FROM DATABASE ok";
      _mediaItem->url      = url;
      _mediaItem->filename = tracksQuery.value(1).toString();
      _mediaItem->title    = tracksQuery.value(2).toString();
      _mediaItem->num      = tracksQuery.value(3).toInt();
      _mediaItem->duration = tracksQuery.value(4).toInt();
      _mediaItem->artist   = tracksQuery.value(5).toString();
      _mediaItem->genre    = tracksQuery.value(6).toString();
      _mediaItem->album    = tracksQuery.value(7).toString();
      _mediaItem->year     = tracksQuery.value(8).toInt();
      return _mediaItem;
    } 
  } // end connect Db

  return 0;
}

//! -------------------------- getCoverName ------------------------------------
QString MEDIA::getCoverName(MediaItem *mi)
{
  QString _coverName = QString();
  
  if(mi) {
    if( (!mi->artist.isEmpty()) && (!mi->album.isEmpty()) ) {
      QCryptographicHash hash(QCryptographicHash::Sha1);
      hash.addData(mi->artist.toLower().toUtf8().constData());
      hash.addData(mi->album.toLower().toUtf8().constData());
      
      _coverName = hash.result().toHex() + ".png";
    }
  }
  
  return _coverName;
}

QString MEDIA:: getCoverName(QString artist, QString album)
{
  QString _coverName = QString();
  
  if( (!artist.isEmpty()) && (!album.isEmpty()) ) {
    QCryptographicHash hash(QCryptographicHash::Sha1);
    hash.addData(artist.toLower().toUtf8().constData());
    hash.addData(album.toLower().toUtf8().constData());
    
    _coverName = hash.result().toHex() + ".png";
  }
  
  return _coverName;
}

  

//! ---------------------- getImageFromMediaUrl --------------------------------
QImage MEDIA::getImageFromMediaUrl(const QString url, QSize size)
{
    TagLib::MPEG::File mpegFile(QFileInfo(url).filePath().toLocal8Bit());
    TagLib::ID3v2::Tag *id3tag = mpegFile.ID3v2Tag(false);
    
    if (!id3tag) {
        return QImage();
    }

    TagLib::ID3v2::AttachedPictureFrame *selectedFrame = MEDIA::attachedPictureFrame(id3tag);
    
    if (!selectedFrame) { // Could occur for encrypted picture frames.
        return QImage();
    }
    
    QByteArray pictureData = QByteArray(selectedFrame->picture().data(), selectedFrame->picture().size());
    QImage attachedImage = QImage::fromData(pictureData);
    
    if(size != attachedImage.size()) {
        attachedImage = attachedImage.scaled(size, Qt::KeepAspectRatio, Qt::SmoothTransformation);
    }
    
    return attachedImage;
}


//! ----------------------- getPixmapFromMediaUrl ------------------------------
QPixmap MEDIA::getPixmapFromMediaUrl(const QString url, QSize size)
{
    return QPixmap::fromImage( getImageFromMediaUrl(url, size) );
}    




//! ---------------------- attachedPictureFrame --------------------------------
TagLib::ID3v2::AttachedPictureFrame *MEDIA::attachedPictureFrame(TagLib::ID3v2::Tag *id3tag, bool create)
{
    // Look for attached picture frames.
    TagLib::ID3v2::FrameList frames = id3tag->frameListMap()["APIC"];
    
    if (frames.isEmpty()) {
        if (create) {
            TagLib::ID3v2::AttachedPictureFrame *selectedFrame = new TagLib::ID3v2::AttachedPictureFrame();
            id3tag->addFrame(selectedFrame);
            return selectedFrame;
        } else {
            return 0;
        }
    }
    
    // According to the spec attached picture frames have different types.
    // So we should look for the corresponding picture depending on what
    // type of image (i.e. front cover, file info) we want.  If only 1
    // frame, just return that (scaled if necessary).
    
    TagLib::ID3v2::AttachedPictureFrame *selectedFrame = 0;
    
    if (frames.size() != 1) {
        TagLib::ID3v2::FrameList::Iterator it = frames.begin();
        for (; it != frames.end(); ++it) {
            
            // This must be dynamic_cast<>, TagLib will return UnknownFrame in APIC for
            // encrypted frames.
            TagLib::ID3v2::AttachedPictureFrame *frame = dynamic_cast<TagLib::ID3v2::AttachedPictureFrame *>(*it);
            
            // Both thumbnail and full size should use FrontCover, as
            // FileIcon may be too small even for thumbnail.
            if (frame && frame->type() != TagLib::ID3v2::AttachedPictureFrame::FrontCover) {
                continue;
            }
            
            selectedFrame = frame;
            break;
        }
    }
    
    // If we get here we failed to pick a picture, or there was only one,
    // so just use the first picture.
    
    if (!selectedFrame) {
        selectedFrame = dynamic_cast<TagLib::ID3v2::AttachedPictureFrame *>(frames.front());
    }
    
    if (!selectedFrame) { // Could occur for encrypted picture frames.
        if (create) {
            TagLib::ID3v2::AttachedPictureFrame *selectedFrame = new TagLib::ID3v2::AttachedPictureFrame();
            id3tag->addFrame(selectedFrame);
            return selectedFrame;
        } else {
            return 0;
        }
    }
    
    return selectedFrame;
}


//! ------------------------ getDurationText -----------------------------------
QString MEDIA::getDurationText(int MediaItemDuration) 
{
 int _durationInt = (MediaItemDuration) > 0 ? MediaItemDuration : 0;
 return QTime(0,0,0,0).addSecs(_durationInt).toString("m:ss");;
}

//! ---------------------- getMediaListDuration --------------------------------
int MEDIA::getMediaListDuration(const QList<MediaItem*> mediaList) 
{
    int listduration = 0;
    for (int i = 0; i < mediaList.count(); i++) {
      	MediaItem *mi = mediaList.at(i);
        if(mi->duration > 0) 
          listduration += mi->duration; 
    }

    return listduration;
}

//! -------------------- getMediaListDurationText ------------------------------
QString MEDIA::getMediaListDurationText(const QList<MediaItem*> mediaList) 
{
    int listduration = MEDIA::getMediaListDuration(mediaList);
    int hours = listduration/3600;
    int minutes = (listduration - (hours*3600))/60;
    int seconds = listduration - (hours*3600) - (minutes*60);
    QString min = minutes < 10 ? QString("0%1").arg(minutes): QString("%1").arg(minutes);
    QString sec = seconds < 10 ? QString("0%1").arg(seconds): QString("%1").arg(seconds);
    
    return QString("%1:%2:%3").arg(hours).arg(min).arg(sec);
}


//! -------------------- isUrlLocal --------------------------------------------
bool MEDIA::isUrlLocal(QString url) 
{
  return (!url.contains(QRegExp("^http://")));
}

bool MEDIA::isMediaItemLocal(MediaItem *mi) 
{
  return MEDIA::isUrlLocal(mi->url);
}
