﻿/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _MEDIAITEM_H_
#define _MEDIAITEM_H_

#include <QtCore>
#include <taglib/id3v2tag.h>
#include <taglib/attachedpictureframe.h>


//! ------ Structure for media item ------ 
struct MediaItem {
    MediaItem() : isPlaying(false), isExists(true) {}

    QString     url        ; //! Url of MediaItem (FULL PATH)
    QString     filename   ; //! Short file name 
    QString     title      ; //! title
    QString     artist     ; //! artist name
    QString     albumArtist; //! artist of album   
    QString     album      ; //! album name
    QString     genre      ; //! genre of media item
    int         duration   ; //! durée
    uint        num        ; //! Numero
    uint        year       ; //! Year

    bool isPlaying;          //! En cours de lecture 
    bool isExists;           //! Flag Fichier Présent
};
  
Q_DECLARE_METATYPE(MediaItem);

  
namespace MEDIA {
  //! ------ Read Tag with Taglib methods -------------------------------------- 
  QPixmap getPixmapFromMediaUrl(const QString url, QSize size = QSize(128,128));
  QImage  getImageFromMediaUrl(const QString url, QSize size);
    
  TagLib::ID3v2::AttachedPictureFrame *
  attachedPictureFrame(TagLib::ID3v2::Tag *id3Tag, bool create = false);

  MediaItem* FromLocalFile(const QString url);
  MediaItem* FromDataBase(const QString url);

  //! ------ CoverName Management ----------------------------------------------
  QString getCoverName(MediaItem *mi);
  QString getCoverName(QString artist, QString album);
  
  //! ------ Media List management --------------------------------------------- 
  QString getDurationText(int MediaItemDuration) ;
  int getMediaListDuration(const QList<MediaItem*> mediaList);
  QString getMediaListDurationText(const QList<MediaItem*> mediaList);

  //! ------ Remote Url management --------------------------------------------- 
  bool isUrlLocal(QString url);
  bool isMediaItemLocal(MediaItem *mi); 
}



#endif // _MEDIAITEM_H_
