/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
 
#include "player.h"

#include <QFileInfo>
#include <phonon/mediaobject.h>
#include <phonon/path.h>
#include <phonon/audiooutput.h>
#include <phonon/mediasource.h>

//! ------------------ Global StatusBar instance -------------------------------
Player* Player::global_player_instance = 0;

namespace GLOBAL
{
    Player* player()
    {
        return Player::instance();
    }
}

//! ------------------ Player --------------------------------------------------
Player::Player()
{
    global_player_instance = this;
  
    _mediaObject = new Phonon::MediaObject();
    _audioOutput = new Phonon::AudioOutput(Phonon::MusicCategory);
    Phonon::createPath(_mediaObject,_audioOutput);

    // By default tick every 1 second
    _mediaObject->setTickInterval(1000);
    
    // Get the next track when there is 2 seconds left on the current one
    // in case of playling track from track view
    _mediaObject->setPrefinishMark( 2000 );
    
    connect(_mediaObject,SIGNAL(finished()),this,SIGNAL(trackFinished()));
    connect(_mediaObject,SIGNAL(stateChanged(Phonon::State,Phonon::State)),this,SIGNAL(stateChanged(Phonon::State,Phonon::State)));
    connect(_mediaObject,SIGNAL(tick(qint64)),this,SIGNAL(trackPositionChanged(qint64)));
    connect(_mediaObject,SIGNAL(totalTimeChanged(qint64)),this,SIGNAL(trackTotalTimeChanged(qint64)));
    
    connect(_mediaObject,SIGNAL(currentSourceChanged(Phonon::MediaSource)),this,SLOT(emitTrackChanged()));
    connect(_mediaObject, SIGNAL(metaDataChanged()), this,SIGNAL(metaDataChanged()));
}

//! ------ set source method ---------------------------------------------------
void Player::setTrack(const QString fileName)
{
  if (QFileInfo(fileName).isFile()) {
    _mediaObject->setCurrentSource(Phonon::MediaSource(fileName));
    _mediaObject->play();  
  }
}

void Player::setTrack(const QUrl url)
{
  if (url.isValid()) {
    _mediaObject->setCurrentSource(Phonon::MediaSource(url));
    //qDebug() << "--- Player --> setTrack " << url.toString();

    _mediaObject->play();  
  }
}

//! ------ current source & metadata -------------------------------------------
QString Player::currentMediaSource()
{
  if (_mediaObject->currentSource().type()==Phonon::MediaSource::Invalid)
    return QString();
    
  return _mediaObject->currentSource().url().toString();
}

const QMap<QString, QString> & Player::currentMetaData() const
{
    return _mediaObject->metaData();
}


//! ------ stop ----------------------------------------------------------------
void Player::stop()
{
    _mediaObject->stop();
    _mediaObject->setCurrentSource(Phonon::MediaSource());
}

void Player::emitTrackChanged()
{
    emit trackChanged(currentMediaSource());
}
