/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _PLAYER_H_
#define _PLAYER_H_

#include <QObject>
#include <QUrl>
#include <QString>
#include <phonon/mediasource.h>
#include <phonon/audiooutput.h>
#include <phonon/mediaobject.h>

class Player;

namespace GLOBAL
{
    Player* player();
}


class Player: public QObject
{ 
Q_OBJECT
  static Player* global_player_instance;

  public:
    Player();

    //! return global instance
    static Player* instance() { return global_player_instance; }
    
    //! method
    QString  currentMediaSource();
    const QMap<QString, QString> &currentMetaData() const;
        
    Phonon::MediaSource currentSource() { return _mediaObject->currentSource(); }
    Phonon::State playerState() { return _mediaObject->state(); }
    QString errorString() { return _mediaObject->errorString(); }
    Phonon::MediaObject* mediaObject() { return _mediaObject; }
    Phonon::AudioOutput* audioOutput() { return _audioOutput; }

 public slots:
    void play() { _mediaObject->play(); }
    void pause() { _mediaObject->pause(); }
    void stop();
    void setTrack(const QString fileName);
    void setTrack(const QUrl url);

 private:
    Phonon::MediaObject   *_mediaObject;
    Phonon::AudioOutput   *_audioOutput;

 private slots:
    void emitTrackChanged();

 signals:
    void trackChanged(QString filename);    // before loading when current soure is set
    void metaDataChanged(); // after loading when metadata resolved
    void trackFinished();
    void stateChanged(Phonon::State newState, Phonon::State oldState);
    void trackPositionChanged(qint64 newPos);
    void trackTotalTimeChanged(qint64 newTotalTime);
};

#endif // _PLAYER_H_
