/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _PLAYLISTMODEL_H
#define _PLAYLISTMODEL_H

#include <QtGui>
#include "MediaItem/mediaitem.h"

//!------------PlaylistDataRoles -----------------------------------------------
namespace Playlist {
  enum PlaylistDataRoles {
        ExistItemRole  = Qt::UserRole + 1,
        PlayingItemRole  = Qt::UserRole + 2
   };
}

//! ----- global instance ------------------------------------------------------
class PlaylistModel;

namespace GLOBAL
{
    PlaylistModel* playlistModel();
}


//!------------PlaylistModel ---------------------------------------------------
class PlaylistModel : public QAbstractListModel 
{
Q_OBJECT
  static PlaylistModel* globalPlaylistModel;

  public:
    PlaylistModel(QWidget *parent);
    
    //! return global instance
    static PlaylistModel* instance() { return globalPlaylistModel; }    
    
    //! inherited from QAbstractListModel
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role) const;
    bool removeRows(int position, int rows, const QModelIndex &parent);

    //! drag and drop
    Qt::ItemFlags flags(const QModelIndex &index) const;
    QStringList mimeTypes() const;
    Qt::DropActions supportedDropActions() const;
    QMimeData* mimeData( const QModelIndexList &indexes ) const;
    bool dropMimeData(const QMimeData *data,
                      Qt::DropAction action, int row, int column,
                      const QModelIndex &parent);

    //! custom methods
    int getActiveRow() {return _activeRow;}
    void setActiveRow(int row);
    void setInactiveMediaItem();
    bool rowExists( int row ) const { return (( row >= 0 ) && ( row < _listMediaItem.size() ) ); }
    void removeIndexes(QModelIndexList &indexes);
    int rowForMediaItem(MediaItem* _mediaItem);
    QModelIndex indexForMediaItem(MediaItem* _mediaItem);
    void move(QModelIndexList &indexes, bool up);

    MediaItem* MediaItemAt( int row ) const;
    MediaItem* getActiveMediaItem() const;
    QString getActiveUrl() const;
    QString getListDuration();
    void emit_layoutChanged();
  
  public slots:
    void addMediaItem(MediaItem* mediaitem);
    void addMediaItemList(QList<MediaItem*> listmediaitem);
    void clear();
    void skipBackward();
    void skipForward();
  
  signals:
    void activeRowChanged();
    void signalAddUrls(const QList<QUrl> listUrl );

  private:
    int                  _activeRow;
    MediaItem*           _activeMediaItem;
    QList<MediaItem*>    _listMediaItem;
};

#endif // _PLAYLISTMODEL_H
