/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "Playlist/playlistpopulator.h"
#include "Playlist/playlistmodel.h"
#include "MediaItem/mediaitem.h"
#include "MediaItem/mediaplaylist.h"
#include "Web/networkaccess.h"
#include "constants.h"


#include "RadioModel/streammodel.h"
#include "RadioModel/streamitem.h"


#include <QDataStream>
#include <QFile>
#include <QByteArray>
#include <QDirIterator>
#include <QMutexLocker>
#include <QDebug>
#include <QFileInfo>


namespace GLOBAL {
    NetworkAccess* http();
}


const QStringList mediaFilter = QStringList() << "mp3" << "ogg" << "flac" << "wav";
const QStringList playlistFilter = QStringList() << "m3u" << "m3u8" << "pls" << "xspf";

//! ------------ PlaylistPopulator class ---------------------------------------
PlaylistPopulator::PlaylistPopulator()
{
    _playlistModel  = PlaylistModel::instance();
    _streamModel    = StreamModel::instance();
    _files.clear();
    _isRunning = false;
}


//! ------------ PlaylistPopulator::run()---------------------------------------
void PlaylistPopulator::run()
{
  _isRunning = true;

  while (!_files.isEmpty()) 
  {
      //! ---- cas des dossiers ---- 
      if (QFileInfo(_files.first()).isDir()) {
        QString dirName = _files.takeFirst();
        QStringList dirFilter  = QStringList() << "*.mp3" << "*.ogg" << "*.flac" << "*.wav";
        QDirIterator dirIterator(dirName, dirFilter ,QDir::Files | QDir::NoSymLinks, QDirIterator::Subdirectories);

        QStringList readfiles; 
        while(dirIterator.hasNext()) {
          readfiles << dirIterator.next(); 
        }
        
        readfiles.append(_files);
        _files = readfiles;
      }
     
      //! ---- cas des playlists locale ---- 
      if (_files.size()>0) {
	//! local playlist
        if( playlistFilter.contains(QFileInfo(_files.first()).suffix().toLower())) {
          if(MEDIA::isUrlLocal(_files.first())) {
              //! local playlist
              QStringList mediaFiles;
              qDebug() << "PlaylistPopulator readPlaylistFile local " << _files.first();
              mediaFiles = MEDIA::readPlaylistFile(_files.takeFirst());
		
              //! put Url into populator list
              if(!mediaFiles.isEmpty()) {
                  qDebug() << "PlaylistPopulator append mediaFiles" << mediaFiles;
                  mediaFiles.append(_files);
                  _files = mediaFiles;   
              }
          }
        }
      }     
      
      //! ---- cas des playlists remote ---- 
      if (_files.size()>0) {
        if( playlistFilter.contains(QFileInfo(_files.first()).suffix().toLower()) ||
	    _files.first().contains("pls?id") /*cas des playlist shoutcast */
	) {
        
            if(!MEDIA::isUrlLocal(_files.first())) {
                //! remote playlist --> download, make it local
                qDebug() << "PlaylistPopulator startDownload" << _files.first();
                QUrl url = QUrl(_files.takeFirst());
                QByteArray bytes = GLOBAL::http()->syncGetBytes(url);
                qDebug() << "PlaylistPopulator QByteArray reveived";
		    
                QFileInfo fileInfo(url.path());
                QString _tempFileName = fileInfo.fileName();
                if (QFile::exists(_CONFIGDIR + "/download/" +  _tempFileName)) {
                  QFile::remove(_CONFIGDIR + "/download/" + _tempFileName);
                }
                QFile *_tempFile =  new QFile(_CONFIGDIR + "/download/" + _tempFileName);
                if (_tempFile->open(QIODevice::WriteOnly)) {
                  QDataStream stream( _tempFile ); 
                  stream.writeRawData(bytes.constData(), bytes.size());
                  _tempFile->close();
                  _files.prepend(_CONFIGDIR + "/download/" + _tempFileName);
                }
              }
        }
      }
      
      //! ---- cas des playlists locale ---- 
      if (_files.size()>0) {
	//! local playlist
        if( playlistFilter.contains(QFileInfo(_files.first()).suffix().toLower())) {
          if(MEDIA::isUrlLocal(_files.first())) {
              //! local playlist
              QStringList mediaFiles;
              qDebug() << "PlaylistPopulator readPlaylistFile local " << _files.first();
              mediaFiles = MEDIA::readPlaylistFile(_files.takeFirst());
		
              //! put Url into populator list
              if(!mediaFiles.isEmpty()) {
                  qDebug() << "PlaylistPopulator append mediaFiles" << mediaFiles;
                  mediaFiles.append(_files);
                  _files = mediaFiles;   
              }
          }
        }
      }    

      //! ---- cas des fichiers et Url ---- 
      if (_files.size() > 0) {
          if(  MEDIA::isUrlLocal(_files.first()) &&
	       mediaFilter.contains(QFileInfo(_files.first()).suffix().toLower()) && 
              !playlistFilter.contains(QFileInfo(_files.first()).suffix().toLower())) {
	      
              QString fileName = _files.takeFirst();
              MediaItem *mi = new MediaItem();
              mi = MEDIA::FromDataBase(fileName);
              if(!mi) mi = MEDIA::FromLocalFile(fileName);
		
              _playlistModel->addMediaItem(mi);
          }
          else if(!MEDIA::isUrlLocal(_files.first())) {
              //! remote file
              qDebug() << "remote uRL" << _files.first();
              MediaItem *mi = new MediaItem();

              //! recherche du stream correspond (si lancé depuis StreamScene)
              StreamItem *st = _streamModel->getStreamItemForUrl(_files.first());
              if(st) {
                qDebug() << "remote uRL from streammodel" << _files.first();
                mi->url        = _files.takeFirst();
                mi->title      = st->name;
                mi->filename   = mi->url;
             }
             else {
               qDebug() << "remote uRL" << _files.first();
               mi->url        = _files.takeFirst();
               mi->title      = mi->url;//QFileInfo(mi->url).baseName();
               mi->filename   = mi->url;
             }
             _playlistModel->addMediaItem(mi);
          }
          else {
              //!WARNING on doit sortir les éléments non traité (si boucle while)
              qDebug() << "media not supported " << _files.takeFirst();
          }
      }
  } //! END !_files.isEmpty()
  
  _isRunning = false;
}

   

//! -------- User methode ------------------------------------------------------
/* 
  addFile  : add one file to proceed
  addFiles : add files
  addUrls  : add Urls
*/
//! -------- PlaylistPopulator::addFile ----------------------------------------
void PlaylistPopulator::addFile(const QString &file)
{
    qDebug() << "append file :" << file;	
    _files.append(file);
}
//! -------- PlaylistPopulator::addFiles ---------------------------------------
void PlaylistPopulator::addFiles(const QStringList &files)
{
    qDebug() << "append files :" << files;	    
    _files.append(files);
}
//! -------- PlaylistPopulator::addUrls ----------------------------------------
void PlaylistPopulator::addUrls(QList<QUrl> listUrl)
{
    foreach (const QUrl &url, listUrl) {
      _files.append(url.toString());
    }
}
