/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _PLAYLISTVIEW_H_
#define _PLAYLISTVIEW_H_

#include "MediaItem/mediaitem.h"
#include "Playlist/playlistmodel.h"

#include <QtGui>
#include <QString>


//!------------PlaylistMode ----------------------------------------------------
// Playlist mode (manage by PlaylistWidget)
namespace Playlist {
  enum PlaylistMode {
      TitleMode      = 1,
      AlbumMode      = 2,
      ArtistMode     = 3
  };
}


//!------------PlaylistView ----------------------------------------------------
class PlaylistDelegate;

class PlaylistView : public QListView
{
  Q_OBJECT
public: 
    PlaylistView(QWidget *parent);
    
    void ClearPlaylist();
    QString GetActiveTrack();
    QString GetNextTrack();
    QString GetPrevTrack();  
    QString GetSelectedTrack();
    void ActiveSelectedTrack();
    void StopPlaying();
    void setPlaylistModel(PlaylistModel *playlistModel);
    PlaylistDelegate* getPlaylistDelegate() {return _playlistdelegate;}
    
    bool isPlaying() {return (_playlistModel->getActiveRow() != -1);}

protected:
  void paintEvent(QPaintEvent *event);
  
private :
    void contextMenuEvent ( QContextMenuEvent * event );

    
private :
    PlaylistModel     *_playlistModel;
    PlaylistDelegate  *_playlistdelegate;
 
    int               _selectedTrack;
    QMenu             *_contextMenu;
    QAction           *removeMediaItemAct;
    
public slots:
    void itemActivated(const QModelIndex &index);
    void selectionChanged (const QItemSelection & selected, const QItemSelection & deselected);
    void updatePlaylistActions();
    
private slots:
    void RemoveTrack();
    
signals :
  void signal_playlist_itemDoubleClicked();
  void signal_playlist_selectionChanged();
};



//!------------PlaylistDelegate ------------------------------------------------
// delegate to have custom painting for track into playlist widget :
//   - paint specific icon for playing item
//   - paint specific icon for broken url item
//   - paint information (artist-album-title) depending on mode
//   - paint duration information
//   - paint simple remote url (with network platform icon)
class PlaylistDelegate : public QItemDelegate
{
public:
  PlaylistDelegate(QObject *parent);
  
  void paint ( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const;
  QSize sizeHint(const QStyleOptionViewItem &option,const QModelIndex &index) const;
  
  Playlist::PlaylistMode   _playlistMode;

private :
   QString getTrackInfo(const MediaItem *mi) const;
   QIcon   getIcon(const MediaItem *mi) const;
};

#endif // _PLAYLISTWIDGET_H_
