/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Playlist/playlistwidget.h" 
#include "Widget/statusbar.h" 
#include "constants.h"

#include <QSettings>


namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}


PlaylistWidget::PlaylistWidget(QWidget *parent,PlaylistView *playlistview)
{
  Q_UNUSED(parent)
  
  //! get global StatusBar instance
  _statusBar = StatusBar::instance();
  
  _playlistdelegate = playlistview->getPlaylistDelegate();

  //! Create Gui
  createGui(playlistview);
  
  //! Playlist Mode
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();

  connect(actions->value("playlistModeTitle"), SIGNAL(triggered()),this,  SLOT(modeTitleTriggered()));
  connect(actions->value("playlistModeAlbum"), SIGNAL(triggered()),this,  SLOT(modeAlbumTriggered()));
  connect(actions->value("playlistModeArtist"), SIGNAL(triggered()),this, SLOT(modeArtistTriggered()));
}

void PlaylistWidget::createGui(PlaylistView *playlistview) 
{
  QVBoxLayout *verticalLayout_0 = new QVBoxLayout(this);
  verticalLayout_0->setSpacing(0);
  verticalLayout_0->setContentsMargins(0, 0, 0, 0);
  playlistview->setFrameShape(QFrame::NoFrame);
  playlistview->setAutoFillBackground(true);  
  setAutoFillBackground(true);  
  verticalLayout_0->addWidget(playlistview);
}

    

    
void PlaylistWidget::setPlaylistModel(PlaylistModel *model) 
{
    _playlistmodel = model;
    connect(_playlistmodel, SIGNAL(layoutChanged()), this, SLOT(updatePlaylistSummary()));
}


//!----------------------------------------------------------------------------
//! save and restore playlist mode
//!----------------------------------------------------------------------------

void PlaylistWidget::saveLastPlaylistMode()
{
    QSettings *_settings = new QSettings(_CONFIGDIR + QDir::separator() + "main.conf",QSettings::IniFormat,this);
    _settings->setValue("Playlist/mode",_playlistMode );
}

void PlaylistWidget::restoreLastPlaylistMode()
{
  QSettings *_settings = new QSettings(_CONFIGDIR + QDir::separator() + "main.conf",QSettings::IniFormat,this);
  
  if(_settings->contains("Playlist/mode"))
    _playlistMode = Playlist::PlaylistMode(_settings->value("Playlist/mode").toInt());
  else 
    _playlistMode = Playlist::TitleMode;
  
  UpdateViewMode(_playlistMode);
}

//!----------------------------------------------------------------------------
//! Mode Switch
//!----------------------------------------------------------------------------

void PlaylistWidget::UpdateViewMode(Playlist::PlaylistMode mode)  
{
   QMap<QString, QAction*> *actions = GLOBAL::globalActions();

  _playlistMode = mode;
  
  ( actions->value("playlistModeTitle"))->setChecked(_playlistMode == Playlist::TitleMode);
  ( actions->value("playlistModeAlbum"))->setChecked(_playlistMode == Playlist::AlbumMode);
  ( actions->value("playlistModeArtist"))->setChecked(_playlistMode == Playlist::ArtistMode);
  
  _playlistdelegate->_playlistMode = _playlistMode;   
  _playlistmodel->emit_layoutChanged();
}


void PlaylistWidget::modeTitleTriggered() 
{
      UpdateViewMode(Playlist::TitleMode);
}

void PlaylistWidget::modeAlbumTriggered()
{
      UpdateViewMode(Playlist::AlbumMode);
}

void PlaylistWidget::modeArtistTriggered()
{
      UpdateViewMode(Playlist::ArtistMode);
}

//!----------------------------------------------------------------------------
//! List Summary
//!----------------------------------------------------------------------------
void PlaylistWidget::updatePlaylistSummary() {

  const int rowCount = _playlistmodel->rowCount(QModelIndex());
  QString duree = _playlistmodel->getListDuration(); 

  const bool isPlaylistEmpty = rowCount < 1;
  QString playlistInfo;
  
  if (isPlaylistEmpty)
    playlistInfo = tr("Empty Playlist");
  else 
    playlistInfo = QString(tr("Playlist : <b>%1</b> tracks - %2")).arg(QString::number(rowCount), duree);

  _statusBar->updatePlaylistInfo(playlistInfo);

}
