/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QtGui>
#include <QtNetwork>

#include "RadioModel/icecastmanager.h"
#include "Web/networkaccess.h"
#include "constants.h"

namespace GLOBAL {
    NetworkAccess* http();
}

//! ------------------ class IceCastManager ------------------------------------  
IceCastManager::IceCastManager() 
{
    qDebug() << " ---- IceCastManager --> creation";
    _isUpdating  = false;
    _isError     = false;
    _downloadTry = 0;
}


QList<StreamItem*> IceCastManager::getIceCastStream()
{
    return readFile();
}


bool IceCastManager::isReady()
{
 //! pour savoir si il faut tlcharger ou non
 QFile file(_CONFIGDIR + "/radio/" + "icecast.xml");
 return (file.exists() && !_isUpdating);
}

bool IceCastManager::isError()
{
 return _isError;
}



//! ------------------ startDownload -------------------------------------------  
void IceCastManager::startDownload() 
{
    qDebug() << " ---- IceCastManager --> startDownload";
    if(_isUpdating) return;
    
    if(_downloadTry++ > 2) {
        _isError = true;
        return;
    }
    _isUpdating = true;
    sendRequest();
}


//! ------------------  sendRequest -------------------------------------------  
void IceCastManager::sendRequest() 
{
  qDebug() << " ---- IceCastManager -> sendRequest";

  QUrl url("http://dir.xiph.org/yp.xml");

  QObject *reply = GLOBAL::http()->get(url);
  connect(reply, SIGNAL(data(QByteArray)), this, SLOT(parseResponse(QByteArray)));
  connect(reply, SIGNAL(error(QNetworkReply*)), this, SLOT(iceCastStop()));
}

//! ------------------  parseResponse -----------------------------------------  
void IceCastManager::parseResponse(QByteArray bytes) 
{
    QString storageLocation = _CONFIGDIR + "/radio/";
    QDir dir;
    dir.mkpath(storageLocation);
    QFile file(storageLocation + "icecast.xml");

    if(file.exists()) file.remove();
  
    if (!file.open(QIODevice::WriteOnly)) {
        emit iceCastStop();
        return;
    }
    // serialize the data into the file
    QDataStream stream( &file ); 
    stream.writeRawData(bytes.constData(), bytes.size());
    
    emit iceCastStop();
}

//! ------------------  readFile -----------------------------------------------  
QList<StreamItem*> IceCastManager::readFile()  
{
    QList<StreamItem*> listStream;
      
    qDebug() << " ---- IceCastManager -> readFile";
    QFile file(_CONFIGDIR + "/radio/" + "icecast.xml");

    if(!file.exists()) return listStream;
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) return listStream;

    QByteArray bytes = file.readAll();
    QXmlStreamReader xml(bytes);
    StreamItem * st ;

    while(!xml.atEnd() && !xml.hasError()) { 
      xml.readNext();
      if (xml.isStartElement()) {
          if(xml.name() == "entry") {
              //! new stream entry
              st = new StreamItem();
          } else if (xml.name() == "server_name") {
              st->name = QString(xml.readElementText());
	      //qDebug() << " ---- IceCastManager -> readFile server_name" << st->name;

          } else if (xml.name() == "listen_url") {
              st->url  = QString(xml.readElementText());
          } else if (xml.name() == "genre") {
              //st->genre = QString(xml.readElementText());
              st->categorie = QString(xml.readElementText());              
          }
      }
      else if (xml.isEndElement()) {
          if(xml.name() == "entry") {
              //! if stream OK --> add it
              if (!st->url.isEmpty() && !st->name.isEmpty()) {
                  listStream.append(st);
              }
          }
      }
    }
    
    qSort(listStream.begin(), listStream.end(), STREAM::compareStreamCategorie);

    return listStream;
}

//! ------------------  iceCastStop --------------------------------------------  
void IceCastManager::iceCastStop()  
{
  qDebug() << " ---- IceCastManager -> iceCastStop";
  _isUpdating = false;
  emit iceCastUpdated();
}
