/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QtGui>
#include <QtNetwork>

#include "RadioModel/shoutcastmanager.h"
#include "RadioModel/streamitem.h"
#include "MediaItem/mediaitem.h"
#include "Web/networkaccess.h"
#include "constants.h"

namespace GLOBAL {
    NetworkAccess* http();
}

//! ------------------ class IceCastManager ------------------------------------  
ShoutCastManager::ShoutCastManager() 
{
    qDebug() << " ---- ShoutCastManager --> creation";
    _isUpdating  = false;
    _isError     = false;
    _downloadTry = 0;
}


QList<StreamItem*> ShoutCastManager::getShoutCastStream()
{
    return readFile();
}


bool ShoutCastManager::isReady()
{
 //! pour savoir si il faut tlcharger ou non
 QFile file(_CONFIGDIR + "/radio/" + "shoutcast.xml");
 return (file.exists() && !_isUpdating);
}

bool ShoutCastManager::isError()
{
 return _isError;
}



//! ------------------ startDownload -------------------------------------------  
void ShoutCastManager::startDownload() 
{
    qDebug() << " ---- ShoutCastManager --> startDownload";
    if(_isUpdating) return;
    
    if(_downloadTry++ > 2) {
        _isError = true;
        return;
    }
    _isUpdating = true;
    sendGenreRequest();
    //sendRequest();
}


void ShoutCastManager::sendRequest() 
{
  _shoutGenreList << "Blues";

    //! clear de la liste des stations
    _stationList.clear();
    foreach (const QString &genre, _shoutGenreList) {
      sendStationRequest(genre);
    }
    writeFile(_stationList);  
    shoutCastStop();
}   
    
//! ------------------  genre request  -----------------------------------------  
void ShoutCastManager::sendGenreRequest() 
{
  qDebug() << " ---- ShoutCastManager -> sendGenreRequest";
  
  //!WARNIG Limit is hard coded = 120
  //QUrl url("http://shoutcast.com/sbin/newxml.phtml");
  QUrl url("http://yp.shoutcast.com/sbin/newxml.phtml");

  QObject *reply = GLOBAL::http()->get(url);
  connect(reply, SIGNAL(data(QByteArray)), this, SLOT(parseGenreResponse(QByteArray)));
  connect(reply, SIGNAL(error(QNetworkReply*)), this, SLOT(shoutCastStopError()));
}

void ShoutCastManager::parseGenreResponse(QByteArray bytes) 
{
    qDebug() << " ---- ShoutCastManager -> parseGenreResponse ";

    QXmlStreamReader xml(bytes);
    _shoutGenreList.clear();

    while(!xml.atEnd() && !xml.hasError()) { 
      xml.readNext();
      if (xml.isStartElement() && xml.name() == "genre") {
        QString genre = xml.attributes().value("name").toString();
        //_shoutGenreList.append(QString(xml.readElementText()));
        _shoutGenreList.append(genre); 
        qDebug() << " ---- ShoutCastManager -> read genre " << genre;
      }
    }
    
    //! clear de la liste des stations
    _stationList.clear();
    foreach (const QString &genre, _shoutGenreList) {
      sendStationRequest(genre);
    }
    writeFile(_stationList);  
    shoutCastStop();
}
//! ------------------  station request ----------------------------------------  
void ShoutCastManager::sendStationRequest(QString genre) 
{
  qDebug() << " ---- ShoutCastManager -> sendStationRequest for genre" << genre;

  QString request = QString("http://yp.shoutcast.com/sbin/newxml.phtml?genre=%1&limit=50").arg(genre);
  QUrl url(request);
  
  QByteArray bytes = GLOBAL::http()->syncGetBytes(url);
  parseStationResponse(bytes, genre);
}

void ShoutCastManager::parseStationResponse(QByteArray bytes, QString &genre) 
{
    QXmlStreamReader xml(bytes);

    while(!xml.atEnd() && !xml.hasError()) { 
      xml.readNext();
      if (xml.isStartElement() && xml.name() == "station") {
        StreamItem * st = new StreamItem();
        QString name = xml.attributes().value("name").toString();
        QString id    = xml.attributes().value("id").toString();
        QString url   = QString("http://yp.shoutcast.com/sbin/tunein-station.pls?id=%1").arg(id);

        st->name      = name;
        st->url       = QUrl(url);
        st->categorie = genre;
	    
        qDebug() << " ---- ShoutCastManager -> added stream" << st->url;
        _stationList.append(st); 
      }
    }
}

//! ------------------  local shoutcast file read/write ------------------------  
void ShoutCastManager::writeFile(QList<StreamItem*> liste)  
{
  //! open file to write
  QFile file(_CONFIGDIR + "/radio/" + "shoutcast.xml");
  if(file.exists()) file.remove();
  if(!file.open(QIODevice::WriteOnly)) return;  
  
  QXmlStreamWriter xml(&file);
  xml.writeStartDocument();
  xml.writeStartElement("shoutcast");
  xml.writeStartElement("trackList");
  
  foreach (StreamItem* si, liste) 
  {
    if (!si) continue;
    
    xml.writeStartElement("track");
    xml.writeTextElement("location", si->url.toString());
    xml.writeTextElement("title", si->name);
    xml.writeStartElement("extension");
    xml.writeAttribute("application", "yarock");
    xml.writeTextElement("category", si->categorie);
    xml.writeEndElement(); //extension
    xml.writeEndElement(); //track
  }
  xml.writeEndElement(); //trackList
  xml.writeEndElement(); //shoutcast
  xml.writeEndDocument(); 
}

QList<StreamItem*> ShoutCastManager::readFile()  
{
    QList<StreamItem*> _listStream;
      
    QFile file(_CONFIGDIR + "/radio/" + "shoutcast.xml");

    if(!file.exists()) return _listStream;
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) return _listStream;

    QXmlStreamReader xml(&file); 
    StreamItem * st = 0;

    while(!xml.atEnd() && !xml.hasError()) { 
      xml.readNext();
      
      if (xml.isStartElement()) {
         if(xml.name() == "track") {
            //! new stream entry
            st = new StreamItem();
         } else if (xml.name() == "title" && st!=0) {
           st->name = QString(xml.readElementText());
         } else if (xml.name() == "location" && st!=0) {
           st->url  = QUrl(xml.readElementText());
         } else if (xml.name() == "category" && st!=0) {
           st->categorie = QString(xml.readElementText());              
         }
      }
      else if (xml.isEndElement() && xml.name() == "track") {
         //! if stream OK --> add it
         if (!st->url.isEmpty() && 
             !MEDIA::isUrlLocal(st->url.toString()) && 
             !st->name.isEmpty()) 
          {
             _listStream.append(st);
          }
          st = 0;
      }
   }
   
    qSort(_listStream.begin(), _listStream.end(), STREAM::compareStreamCategorie);
    return _listStream;
}


//! ------------------  shoutCastStop ------------------------------------------  
void ShoutCastManager::shoutCastStop()  
{
  qDebug() << " ---- ShoutCastManager -> shoutCastStop";
  _isUpdating = false;
  emit shoutCastUpdated();
}

void ShoutCastManager::shoutCastStopError()  
{
  qDebug() << " ---- ShoutCastManager -> shoutCastStopError";
  _isUpdating = false;
  emit shoutCastStoppedError();
}
