/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "RadioModel/streammodel.h"

//! ----- global instance ------------------------------------------------------
StreamModel* StreamModel::globalStreamModel = 0;

namespace GLOBAL
{
    StreamModel* streamModel()
    {
        return StreamModel::instance();
    }
}


//! ------------------ class StreamModel --------------------------------------  
StreamModel::StreamModel(QObject *parent) : QAbstractTableModel(parent) 
{
    globalStreamModel = this;
    
    qDebug() << " ---- StreamModel --> creation";
    QHash<int, QByteArray> roleNames;
    roleNames.insert(Qt::UserRole + StreamUrl, "streamUrl");
    roleNames.insert(Qt::UserRole + StreamName, "name");
    roleNames.insert(Qt::UserRole + StreamGenre, "genre");
    setRoleNames(roleNames);    
    
    modelUpdated     = false;
    _filterPattern   = "";
}

void StreamModel::clear()
{
  qDeleteAll(_listStream.constBegin(), _listStream.constEnd());
  _listStream.clear();
}

//! ------------------ StreamModel::updateListeStream --------------------------
void StreamModel::updateListeStream(QList<StreamItem*> liste)
{
    qDebug() << " ---- StreamModel --> updateListeStream START";
    if (!liste.empty()) 
    {
      beginInsertRows(QModelIndex(), this->_listStream.size(), this->_listStream.size() + liste.size() - 1);
      _listStream.append(liste);
      endInsertRows();
    }

    modelUpdated = true;
    qDebug() << " ---- StreamModel --> updateListeStream END";
}

//!------------StreamModel::rowCount -------------------------------------------
int StreamModel::rowCount(const QModelIndex & /* parent */) const 
{
    return _listStream.size();
}

//!------------StreamModel::columnCount ----------------------------------------
int StreamModel::columnCount(const QModelIndex & /* parent */) const 
{
    return MaxColumns;
}

//!------------StreamModel::data -----------------------------------------------
QVariant StreamModel::data(const QModelIndex &index, int role) const
{
Q_UNUSED(role)
    const int row = index.row();
    if (row < 0 || row >= _listStream.size())
        return QVariant();

    StreamItem *stream = _listStream.at(row);
    if (!stream) return QVariant();
    
    switch (index.column()) {
      case StreamUrl:   return stream->url;break;
      case StreamName:  return stream->name;break;
      case StreamGenre: return stream->genre;break;
      default: return QVariant();break;
    }

    return QVariant();
}

QVariant StreamModel::headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const
{
    if (role != Qt::DisplayRole || orientation != Qt::Horizontal) 
        return QVariant();
    
    switch (section) {
      case StreamUrl:   return tr("Url");
      case StreamName:  return tr("Name");
      case StreamGenre: return tr("Genre");
      default: return QVariant();
    }
}

//!------------StreamModel::getStreamItem ---------------------------------------
StreamItem * StreamModel::getStreamItem(int row) 
{
   if (row < 0 || row >= _listStream.size())
        return new StreamItem();

   return _listStream.at(row);
}

StreamItem * StreamModel::getStreamItemForUrl(const QString urlName)
{
 for ( int i = 0; i < _listStream.size(); i++ )
  {
     StreamItem * st = _listStream.at(i);
     if (urlName == st->url.toString()) 
       return st;
  }
  return 0;
}
//! --------- playing item method ----------------------------------------------
void StreamModel::updatePlayingItem(const QString urlName)
{
  qDebug() << " ---- StreamModel --> updatePlayingItem" << urlName;

  for ( int i = 0; i < _listStream.size(); i++ )
  {
     StreamItem * st = _listStream.at(i);
     if (urlName == st->url.toString()) 
     {
       st->isPlaying         = true;
       emit signalPlayingStatusChanged();
       break;
     }
  }
}

void StreamModel::stopPlayingItem()
{
  for ( int i = 0; i < _listStream.size(); i++ )
  {
     StreamItem * st = _listStream.at(i);
     if (st->isPlaying) {
       st->isPlaying = false;
       emit signalPlayingStatusChanged();
       break;
    }
  }
}  
  
//! --------- Filtering method -------------------------------------------------
bool StreamModel::isStreamFiltered(const int row)
{
  StreamItem *st = getStreamItem(row);
  if (st->name.contains(_filterPattern, Qt::CaseInsensitive) ||
      st->genre.contains(_filterPattern, Qt::CaseInsensitive))
      return true;
  
  return false;
}
