/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef _STREAMMODEL_H_
#define _STREAMMODEL_H_

#include <QList>
#include <QAbstractTableModel>
#include "RadioModel/streamitem.h"

static const int MaxColumns = 3;

class StreamModel;

namespace GLOBAL
{
    StreamModel* streamModel();
}



class StreamModel  : public QAbstractTableModel
{
Q_OBJECT
  static StreamModel* globalStreamModel;

  public:
    StreamModel(QObject *parent);
    
    //! return global instance
    static StreamModel* instance() { return globalStreamModel; }    
     
    //! liste streal update
    void updateListeStream(QList<StreamItem*> liste);

    //! QAbstractTableModel implementation
    enum Column {StreamUrl, StreamName, StreamGenre};
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    int columnCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
    QVariant headerData(int section, Qt::Orientation orientation, int role) const;

    //! get Stream Item method
    StreamItem * getStreamItem(int row);
    StreamItem * getStreamItemForUrl(const QString urlName);

    //! playing status method
    void     stopPlayingItem();
    void     updatePlayingItem(const QString urlName);
    
    //! filtering method
    QString               _filterPattern;
    bool isStreamFiltered(const int row);
    
    void clear();

  public:
    bool                  modelUpdated;

  private:
   QList<StreamItem*>   _listStream;

signals:
    void signalPlayingStatusChanged();
};

#endif // _STREAMMODEL_H_ 
