/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QtGui>
#include <QUrl>
#include <QtNetwork>

#include "RadioModel/xspfstreamfile.h"
#include "MediaItem/mediaitem.h"
#include "constants.h"

//! --------------------------------------------------------------------
QList<StreamItem*> STREAM::readXspfStreamFile(QString filename)  
{
    QList<StreamItem*> _listStream;
      
    qDebug() << " ---- CustomStreamReader -> readXspfFile  filename" << filename;
    QFile file(_CONFIGDIR + "/radio/" + filename);

    if(!file.exists()) return _listStream;
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) return _listStream;

    QXmlStreamReader xml(&file); 
    StreamItem * st = 0;

    while(!xml.atEnd() && !xml.hasError()) { 
      xml.readNext();
      
      if (xml.isStartElement()) {
	if(xml.name() == "track") {
	  //! new stream entry
	  st = new StreamItem();
	} else if (xml.name() == "title" && st!=0) {
	  st->name = QString(xml.readElementText());
	} else if (xml.name() == "location" && st!=0) {
	  st->url  = QUrl(xml.readElementText());
	} else if (xml.name() == "category" && st!=0) {
	  st->categorie = QString(xml.readElementText());              
	}
      }
      else if (xml.isEndElement() && xml.name() == "track") {
              //! if stream OK --> add it
              if (!st->url.isEmpty() && !st->name.isEmpty()) {
                  _listStream.append(st);
              }
              st = 0;
          }
      }
   
    qSort(_listStream.begin(), _listStream.end(), STREAM::compareStreamCategorie);
    return _listStream;
}

//! --------------------------------------------------------------------
QList<StreamItem*> STREAM::readXspfStreamFavorite()  
{
    QList<StreamItem*> _listStream;
      
    QFile file(_CONFIGDIR + "/favorite.xspf");

    if(!file.exists()) return _listStream;
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) return _listStream;

    QXmlStreamReader xml(&file); 
    StreamItem * st = 0;

    while(!xml.atEnd() && !xml.hasError()) { 
      xml.readNext();
      
      if (xml.isStartElement()) {
	if(xml.name() == "track") {
	  //! new stream entry
	  st = new StreamItem();
	} else if (xml.name() == "title" && st!=0) {
	  st->name = QString(xml.readElementText());
	} else if (xml.name() == "location" && st!=0) {
	  st->url  = QUrl(xml.readElementText());
	} else if (xml.name() == "category" && st!=0) {
	  st->categorie = QString(xml.readElementText());              
	}
      }
      else if (xml.isEndElement() && xml.name() == "track") {
              //! if stream OK --> add it
              if (!st->url.isEmpty() && 
                  !MEDIA::isUrlLocal(st->url.toString()) && 
                  !st->name.isEmpty()) {
                  _listStream.append(st);
              }
              st = 0;
          }
      }
   
    qSort(_listStream.begin(), _listStream.end(), STREAM::compareStreamCategorie);
    return _listStream;
}

//! --------------------------------------------------------------------
void STREAM::saveXspfStreamFavorite(QList<StreamItem*> _listStreamItem) 
{
  //! open file to write
  QFile file(_CONFIGDIR + "/favorite.xspf");  
  if(file.exists()) file.remove();
  if(!file.open(QIODevice::WriteOnly)) return;  
  
  QXmlStreamWriter xml(&file);
  xml.writeStartDocument();
  xml.writeStartElement("playlist");
  xml.writeAttribute("version", "1");
  xml.writeDefaultNamespace("http://xspf.org/ns/0/");
  xml.writeStartElement("trackList");
  
  foreach (StreamItem* si, _listStreamItem) 
  {
    if (!si) continue;
    
    xml.writeStartElement("track");
    xml.writeTextElement("location", si->url.toString());
    xml.writeTextElement("title", si->name);
    xml.writeStartElement("extension");
    xml.writeAttribute("application", "yarock");
    xml.writeTextElement("category", si->categorie);
    xml.writeEndElement(); //extension
    xml.writeEndElement(); //track
  }
  xml.writeEndElement(); //trackList
  xml.writeEndElement(); //playlist
  xml.writeEndDocument(); 
}
