/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "RadioView/radioscene.h"
#include "RadioModel/streammodel.h"
#include "RadioModel/streamitem.h"
#include "RadioModel/icecastmanager.h"
#include "RadioModel/xspfstreamfile.h"
#include "MediaItem/mediaitem.h"
#include "MediaItem/mediaplaylist.h"
#include "Web/networkaccess.h"
#include "constants.h"

namespace GLOBAL {
    NetworkAccess* http();
}

namespace GLOBAL {
    QMap<QString, QAction*>* globalActions();
}


//! --------------------- RadioScene constructor ------------------------------
RadioScene::RadioScene(QWidget *parent)
{
  setParent(parent);
  _parentWidget = parent;
  setItemIndexMethod(QGraphicsScene::NoIndex);
    
  _infoSize        = 0;
 
  //! set model  
  _streamModel = StreamModel::instance();
  connect(_streamModel, SIGNAL(signalPlayingStatusChanged()), this, SLOT(update()));

  //! set player  
  _player = Player::instance();
  connect(_player, SIGNAL(trackChanged(QString)), this, SLOT(updateStreamPlaying(QString))); 
  
  //! ice cast
  _iceCastManager = new IceCastManager();
  connect(_iceCastManager, SIGNAL(iceCastUpdated()), this, SIGNAL(repopulateRadioScene())); 
  
  //! ice cast
  _shoutCastManager = new ShoutCastManager();
  connect(_shoutCastManager, SIGNAL(shoutCastUpdated()), this, SIGNAL(repopulateRadioScene())); 
  
  _radioMode           = Radio::ViewIceCast;
  _playlistPlayling    = "";
  _mouseGrabbedItem    = 0;
  
  //! item double click 
  QAction *streamDoubleClick = new QAction(this); 
  QObject::connect(streamDoubleClick, SIGNAL(triggered()), this, SLOT(streamDoubleClick()), Qt::DirectConnection);
  QMap<QString, QAction*> *actions = GLOBAL::globalActions();  
  actions->insert("streamDoubleClick", streamDoubleClick);  
}   

RadioScene::~RadioScene()
{
  clear(); 
}   

void RadioScene::setVirtualPlaylist(VirtualPlaylist  *playlist)
{
  _virtualPlaylist = playlist;
  _virtualPlaylist->StopPlaying();
}

//! --------------------- playing status method --------------------- 
void RadioScene::updateStreamPlaying(QString urlName)
{  
  //qDebug() << " ---- RadioScene -> updateStreamPlaying" << urlName;
  //qDebug() << " ---- RadioScene -> updateStreamPlaying _playlistPlayling" << _playlistPlayling;
  if(!urlName.isEmpty()) 
  {
    if  (!MEDIA::isUrlLocal(urlName)) {
        _streamModel->stopPlayingItem();
        if(_playlistPlayling.isEmpty())
          _streamModel->updatePlayingItem(urlName);
        else
          _streamModel->updatePlayingItem(_playlistPlayling);
    }
  }
  else
  {  
     _playlistPlayling = "";
     _streamModel->stopPlayingItem();
    _virtualPlaylist->StopPlaying();
  }
}


//! --------------------- playing method ---------------------------
const QStringList playlistFilter = QStringList() << "m3u" << "m3u8" << "pls" << "xspf";

void RadioScene::playStream() 
{
  qDebug() << " ---- RadioScene -> playStream";
  if(!_mouseGrabbedItem) return;
  StreamGraphicItem *item = static_cast<StreamGraphicItem*>(_mouseGrabbedItem);
  
  StreamItem *st = _streamModel->getStreamItem(item->_StreamRow);
  QString streamUrl = st->url.toString();

  qDebug() << " ---- RadioScene -> playStream  QFileInfo(streamUrl).filename()" << QFileInfo(streamUrl).fileName() ;

  if( !playlistFilter.contains(QFileInfo(streamUrl).suffix().toLower()) &&
       !streamUrl.contains("pls?id")
  ) {
    qDebug() << " ---- RadioScene -> playStream  we can play it's an url"<< streamUrl;

    _playlistPlayling = "";
    _virtualPlaylist->AppendFileInPlaylist(streamUrl);
  }
  else 
  {
    qDebug() << " ---- RadioScene -> playStream  we can not play --> download to do" ;

    //! start a synchronous download
    QByteArray bytes = GLOBAL::http()->syncGetBytes(st->url);
	
    QFileInfo fileInfo(st->url.path());
    QString _tempFileName = fileInfo.fileName();
    qDebug() << " ---- RadioScene -> playStream  we can not play --> _tempFileName" << _tempFileName ;

    if (QFile::exists(_CONFIGDIR + "/download/" +  _tempFileName)) {
      QFile::remove(_CONFIGDIR + "/download/" + _tempFileName);
    }
    QFile *_tempFile =  new QFile(_CONFIGDIR + "/download/" + _tempFileName);
    if (_tempFile->open(QIODevice::WriteOnly)) {
      QDataStream stream( _tempFile ); 
      stream.writeRawData(bytes.constData(), bytes.size());
      _tempFile->close();
    }
    
    QStringList list = MEDIA::readPlaylistFile(_CONFIGDIR + "/download/" + _tempFileName);
    if(!list.isEmpty()) {
      _playlistPlayling = streamUrl;
      _virtualPlaylist->AppendFileInPlaylist(list.first());
    }
    else {
      _playlistPlayling = "";
    }
  }
}    

//! --------------------- RadioScene Populator ---------------------------
void RadioScene::populateScene() 
{
  qDebug() << " ---- RadioScene -> PopulateScene ----";
  
  //! clear scene and delete all item
  clear();
  switch(_radioMode) {
    case (Radio::ViewIceCast) : 
      if(_iceCastManager->isReady()) {
        qDebug() << " ---- RadioScene -> icecastready ----";
	_streamModel->clear();
        _streamModel->updateListeStream(_iceCastManager->getIceCastStream());
        populateStreamScene(); 
        updateStreamPlaying(_player->currentMediaSource());
      }
      else {
        qDebug() << " ---- RadioScene -> icecast startdownload ----";
        _iceCastManager->startDownload();
        populateWaitingScene("Start downloading icecast stream....please wait...");
      }    
      break;
    
    case (Radio::ViewShoutCast) : 
      if(_shoutCastManager->isReady()) {
        qDebug() << " ---- RadioScene -> shoutcastready ----";
	_streamModel->clear();
        _streamModel->updateListeStream(_shoutCastManager->getShoutCastStream());
        populateStreamScene(); 
        updateStreamPlaying(_player->currentMediaSource());
      }
      else {
        qDebug() << " ---- RadioScene -> shoutcast startdownload ----";
        _shoutCastManager->startDownload();
        populateWaitingScene("Start downloading shoutcast stream....please wait...");
      }    
      break;
      
    case (Radio::ViewCustomRadio) :
        _streamModel->clear();
        _streamModel->updateListeStream(STREAM::readXspfStreamFile(currentXspfFilename));
        populateStreamScene();
        updateStreamPlaying(_player->currentMediaSource());
    break;
      
    case (Radio::ViewFavoriteRadio) :
        _streamModel->clear();
        _streamModel->updateListeStream(STREAM::readXspfStreamFavorite());
        populateFavoriteStreamScene();
        updateStreamPlaying(_player->currentMediaSource());
    break;
    
    default: break;
  } 
  
  //! we need to ajust SceneRect 
  int diff = _parentWidget->height() - itemsBoundingRect().height() + 40;
  int ajust  = ( diff > 0) ? diff : 0; 
  setSceneRect ( itemsBoundingRect().adjusted(-10, -10, +10, +10 + ajust) );
}



void RadioScene::populateStreamScene() 
{
  qDebug() << " ----RadioScene -> populateStreamScene ----";
  
  int streamRow      = 0;
  int categoriesRow  = 0;
  _infoSize          = 0;
  
  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  QString currentCategoryName;
  //! stream model loop
  for ( int i = 0; i < _streamModel->rowCount (); i++ ) 
  {
    if(!_streamModel->isStreamFiltered(i)) continue;
    
    if(currentCategoryName != _streamModel->getStreamItem(i)->categorie) {
      //! new category
      currentCategoryName = _streamModel->getStreamItem(i)->categorie;
     
      CategorieGraphicItem *_cat = new CategorieGraphicItem();
      _cat->_catName = currentCategoryName;
      _cat->setPos( 10 ,categoriesRow*40 + streamRow*20 );
      addItem(_cat);
      categoriesRow++;
      _listCategorie.append(_cat);
    }

    StreamGraphicItem *_stream = new StreamGraphicItem();
    _stream->isFavorite = false;
    _stream->_StreamRow = i;
    _stream->setPos(180, categoriesRow*40 + streamRow*20);
    addItem(_stream); 
 
    streamRow++;
    _infoSize++;
  }  
}

void RadioScene::populateFavoriteStreamScene() 
{
  qDebug() << " ----RadioScene -> populateStreamScene ----";
  
  int streamRow      = 0;
  int categoriesRow  = 0;
  _infoSize          = 0;
  
  //! categories list (for up/down navigation)
  _listCategorie.clear();
  
  QString currentCategoryName;
  //! stream model loop
  for ( int i = 0; i < _streamModel->rowCount (); i++ ) 
  {
    if(!_streamModel->isStreamFiltered(i)) continue;
    
    if(currentCategoryName != _streamModel->getStreamItem(i)->categorie) {
      //! new category
      currentCategoryName = _streamModel->getStreamItem(i)->categorie;
     
      CategorieGraphicItem *_cat = new CategorieGraphicItem();
      _cat->_catName = currentCategoryName;
      _cat->setPos( 10 ,categoriesRow*40 + streamRow*20 );
      addItem(_cat);
      categoriesRow++;
      _listCategorie.append(_cat);
    }

    StreamGraphicItem *_stream = new StreamGraphicItem();
    _stream->isFavorite = true;
    _stream->_StreamRow = i;
    _stream->setPos(180, categoriesRow*40 + streamRow*20);
    addItem(_stream); 
   
    streamRow++;
    _infoSize++;
  }  
}

void RadioScene::populateWaitingScene(QString text) 
{
  qDebug() << " ----RadioScene -> populateWaitingScene ----";
   _listCategorie.clear();
   
  QGraphicsTextItem *item = new QGraphicsTextItem(text);
  item->setPos( 10 ,40);  
  addItem(item);
}

//! --------------------- Favorite Methods --------------------- 
void RadioScene::addStreamToFavorite() 
{
  qDebug() << " ---- RadioScene -> addStreamToFavorite";
  if(!_mouseGrabbedItem) return;
  StreamGraphicItem *item = static_cast<StreamGraphicItem*>(_mouseGrabbedItem);
  
  bool exist = false;

  StreamItem *streamToAdd = _streamModel->getStreamItem(item->_StreamRow);

  //! get current favorite
  QList <StreamItem *> currentList = STREAM::readXspfStreamFavorite();
  
  //! add stream if does not exist 
  foreach (StreamItem* st, currentList) {
    if(st->url.toString() == streamToAdd->url.toString()) {
      exist = true;
      break;
    }
  }
  
  if(!exist) {
    currentList.append(streamToAdd);
    qSort(currentList.begin(), currentList.end(), STREAM::compareStreamCategorie);

    //! save favorite
    STREAM::saveXspfStreamFavorite(currentList);
  }
}  


void RadioScene::removeStreamToFavorite() 
{
  qDebug() << " ---- RadioScene -> removeStreamToFavorite";
  if(!_mouseGrabbedItem) return;
  StreamGraphicItem *item = static_cast<StreamGraphicItem*>(_mouseGrabbedItem);

  bool removed = false;

  StreamItem *streamToRemove = _streamModel->getStreamItem(item->_StreamRow);

  //! get current favorite
  QList <StreamItem *> currentList = STREAM::readXspfStreamFavorite();
  
  //! add stream if does not exist 
  foreach (StreamItem* st, currentList) {
    if(st->url.toString() == streamToRemove->url.toString()) {
      currentList.removeAt(currentList.indexOf(st));
      removed = true;
      break;
    }
  }
  
  if(removed) {
    //! save favorite
    qSort(currentList.begin(), currentList.end(), STREAM::compareStreamCategorie);
    STREAM::saveXspfStreamFavorite(currentList);
    populateScene();
  }
}  


//! --------------------- contextMenuEvent ------------------------------------- 
void RadioScene::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) 
{
  _mouseGrabbedItem = this->mouseGrabberItem();
  
  if(!_mouseGrabbedItem) {
      //! TODO implementer add stream + reload streamlist
      //QAction *addStreamAct        = new QAction(tr("Add a new stream to the list"),this);
      QAction *reloadStreamListAct = new QAction(QIcon(":/images/rebuild.png"),tr("Reload stream list"),this);
      QObject::connect(reloadStreamListAct, SIGNAL(triggered()), this, SLOT(reloadList()), Qt::DirectConnection);

      QMenu *streamMenu      = new QMenu();
      //streamMenu->addAction(addStreamAct);
      streamMenu->addAction(reloadStreamListAct);
      streamMenu->exec(event->screenPos());
  }
  else if (_mouseGrabbedItem->type()== GraphicsItem::StreamType) {
      StreamGraphicItem *item = static_cast<StreamGraphicItem*>(_mouseGrabbedItem);

      //! Actions
      QAction *streamPlayAct = new QAction(QIcon(":/images/media-play.png"), tr("Play Stream"), this);
      QObject::connect(streamPlayAct, SIGNAL(triggered()), this, SLOT(playStream()), Qt::DirectConnection);
      QAction *favoriteStreamAct = new QAction(QIcon(":/images/media-favorite.png"), tr("Add stream to favorites"), this);

      //! TODO
      //QAction *removeStreamAct = new QAction(QIcon::fromTheme("edit-delete"),tr("Remove stream from list"),this);
      //QObject::connect(removeStreamAct, SIGNAL(triggered()), this, SLOT(removeStreamFromList()), Qt::DirectConnection);
  
      if(item->isFavorite) {    
        favoriteStreamAct->setText (tr("Remove Stream from favorites"));
        QObject::connect(favoriteStreamAct, SIGNAL(triggered()), this, SLOT(removeStreamToFavorite()), Qt::DirectConnection);
      }
      else {
        QObject::connect(favoriteStreamAct, SIGNAL(triggered()), this, SLOT(addStreamToFavorite()), Qt::DirectConnection);
      }
      QMenu *streamMenu      = new QMenu();
      streamMenu->addAction(streamPlayAct);
      streamMenu->addSeparator();
      streamMenu->addAction(favoriteStreamAct);
      streamMenu->exec(event->screenPos());
  } 
}


//! --------------------- itemDoubleClick ------------------------------------- 
void RadioScene::streamDoubleClick()
{
   qDebug() << " ---- RadioScene::itemDoubleClick";
   QMap<QString, QAction*> *actions = GLOBAL::globalActions();
  
   qDebug() << " ---- RadioScene::itemDoubleClick" << (actions->value("streamDoubleClick"))->data();
   
   QGraphicsItem *gItem = qvariant_cast<QGraphicsItem *>( (actions->value("streamDoubleClick"))->data() );
   _mouseGrabbedItem = gItem;
   if(!_mouseGrabbedItem) return;
   
   switch(gItem->type()) {
     case GraphicsItem::StreamType: playStream();break;
     default:break;    
   }
}

//! --------------------- reloadList ------------------------------------------
void RadioScene::reloadList()
{
    if(_radioMode == Radio::ViewIceCast) {
       QFile file(_CONFIGDIR + "/radio/" + "icecast.xml");
       if(file.exists() && _iceCastManager->isReady()) file.remove();
    }
    else if(_radioMode == Radio::ViewShoutCast) {
       QFile file(_CONFIGDIR + "/radio/" + "shoutcast.xml");
       if(file.exists() && _shoutCastManager->isReady()) file.remove();
    }

    populateScene();
}

