/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _RADIO_SCENE_H_
#define _RADIO_SCENE_H_

#include "RadioModel/streammodel.h"
#include "RadioModel/icecastmanager.h"
#include "RadioModel/shoutcastmanager.h"
#include "RadioModel/xspfstreamfile.h"
#include "CollectionView/virtualplaylist.h"
#include "CollectionView/graphicitem.h"
#include "Player/player.h"

#include <QStringList>
#include <QGraphicsScene>
#include <QtGui>

namespace Radio {
  enum RadioMode {
      ViewIceCast           = 1,
      ViewShoutCast         = 2,
      ViewCustomRadio       = 3,
      ViewFavoriteRadio     = 4
  };
}


//!----------------------------------------------------------------------------
//! RadioScene
//!----------------------------------------------------------------------------
class RadioScene : public QGraphicsScene
{
Q_OBJECT
  public: 
    RadioScene(QWidget *parent);
    ~RadioScene();
    void  setVirtualPlaylist(VirtualPlaylist  *playlist);
    int   getSceneInfoSize() {return _infoSize;}

    //! data model 
    StreamModel                   *_streamModel;
    QList<CategorieGraphicItem*>  _listCategorie;    
    Radio::RadioMode              _radioMode;
    QString                      currentXspfFilename;
    
  private :
    IceCastManager               *_iceCastManager;
    ShoutCastManager             *_shoutCastManager;

    VirtualPlaylist              *_virtualPlaylist;
    Player                       *_player;
    int                          _infoSize;
    QWidget                      *_parentWidget;
    QString                      _playlistPlayling;
    QGraphicsItem                *_mouseGrabbedItem;
    
  protected:
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
    
  private slots :
    void populateStreamScene();
    void populateFavoriteStreamScene();
    void populateWaitingScene(QString text);
    void playStream();
    void addStreamToFavorite();
    void removeStreamToFavorite();
    void streamDoubleClick();
    void reloadList();

  public slots:
    void populateScene();
    void updateStreamPlaying(QString urlName);

signals:
    void repopulateRadioScene();
};

#endif // _RADIO_SCENE_H_
