/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "settingspages.h"
#include "constants.h"

#include <QFileDialog>
#include <QListWidgetItem>
#include <QDebug>


//! ----------------------------------------------------------------------------
//! Global Settings Page  
//! ----------------------------------------------------------------------------
SettingGlobalPage::SettingGlobalPage(QWidget *parent) : QWidget(parent)
{
  //! create Gui
  createGui();
}

void SettingGlobalPage::createGui()
{
    QVBoxLayout* vl0 = new QVBoxLayout(this);
    vl0->setObjectName(QString::fromUtf8("verticalLayout"));

    //! Check box for Minimaze App to Systray option
    minimazeToSystray = new QCheckBox(this);
    minimazeToSystray->setObjectName(QString::fromUtf8("minimazeToSystray"));
    minimazeToSystray->setText(tr("Minimize application to systray"));
    vl0->addWidget(minimazeToSystray);
    
    QSpacerItem* vspacer0 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);
    vl0->addItem(vspacer0);    
}


//! ----------------------------------------------------------------------------
//! Collection Settings Page
//! ----------------------------------------------------------------------------
SettingCollectionPage::SettingCollectionPage(QWidget *parent) : QWidget(parent)
{
  //! create Gui
  createGui();

  //! connection
  connect(this->addPathButton_2, SIGNAL(clicked()), this, SLOT(on_addPathButton_clicked()));
  connect(this->removePathButton_2, SIGNAL(clicked()), this, SLOT(on_removePathButton_clicked()));
  connect(this->removeAllPathsButton_2, SIGNAL(clicked()), this, SLOT(on_removeAllPathsButton_clicked()));

  //! default --> no change
  _collectionsSourceChanged = false;
}

//! ----------- createGui ------------------------------------------------------  
void SettingCollectionPage::createGui()
{
    QVBoxLayout* verticalLayout = new QVBoxLayout(this);
    verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
     
    QHBoxLayout* mainHorizontalLayout = new QHBoxLayout();
    mainHorizontalLayout->setObjectName(QString::fromUtf8("mainHorizontalLayout"));
     
    collectionsPathsList = new QListWidget(this);
    collectionsPathsList->setObjectName(QString::fromUtf8("collectionsPathsList"));
    mainHorizontalLayout->addWidget(collectionsPathsList);
     
    //! Vertical layout for button Remove/Remove All/Add
    QVBoxLayout *verticalLayout_3 = new QVBoxLayout();
    verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
    
    addPathButton_2 = new QPushButton(this);
    addPathButton_2->setObjectName(QString::fromUtf8("addPathButton_2"));
    addPathButton_2->setText(tr("Add ..."));
    
    verticalLayout_3->addWidget(addPathButton_2);
  
    removePathButton_2 = new QPushButton(this);
    removePathButton_2->setObjectName(QString::fromUtf8("removePathButton_2"));
    removePathButton_2->setText(tr("Remove"));
  
    verticalLayout_3->addWidget(removePathButton_2);
  
    removeAllPathsButton_2 = new QPushButton(this);
    removeAllPathsButton_2->setObjectName(QString::fromUtf8("removeAllPathsButton_2"));
    removeAllPathsButton_2->setText(tr("Remove all"));
  
    verticalLayout_3->addWidget(removeAllPathsButton_2);
  
    QSpacerItem* verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);
  
    verticalLayout_3->addItem(verticalSpacer_2);
    mainHorizontalLayout->addLayout(verticalLayout_3);

    verticalLayout->addLayout(mainHorizontalLayout);

    //! Check box auto check database
    autoupdateCollectionsCheckbox = new QCheckBox(this);
    autoupdateCollectionsCheckbox->setObjectName(QString::fromUtf8("autoupdateCollectionsCheckbox"));
    autoupdateCollectionsCheckbox->setText(tr("Update collections automatically after start"));
    verticalLayout->addWidget(autoupdateCollectionsCheckbox);
  
    //! Check box --> search cover into directory
    searchCoverFromDir = new QCheckBox(this);
    searchCoverFromDir->setObjectName(QString::fromUtf8("searchCoverFromDir"));
    searchCoverFromDir->setText(tr("Search cover art from file directory"));
    verticalLayout->addWidget(searchCoverFromDir);
}


//! ----------- on_addPathButton_clicked ---------------------------------------  
void SettingCollectionPage::on_addPathButton_clicked()
{
    QString dirName = QFileDialog::getExistingDirectory(this,
                      tr("Add directory"),
                      QString(),
                      QFileDialog::ShowDirsOnly);
    if (!dirName.isEmpty()) {
        collectionsPathsList->addItem(dirName);
	_collectionsSourceChanged = true;
    }
    
}

//! ----------- on_removePathButton_clicked ------------------------------------  
void SettingCollectionPage::on_removePathButton_clicked()
{
    foreach (QListWidgetItem *item, collectionsPathsList->selectedItems()) {
        delete collectionsPathsList->takeItem(collectionsPathsList->row(item));
    }
    _collectionsSourceChanged = true;
}

//! ----------- on_removeAllPathsButton_clicked --------------------------------  
void SettingCollectionPage::on_removeAllPathsButton_clicked()
{
    collectionsPathsList->clear();
    _collectionsSourceChanged = true;
}



//! ----------------------------------------------------------------------------
//! Radio Settings Page
//! ----------------------------------------------------------------------------
SettingRadioPage::SettingRadioPage(QWidget *parent) : QWidget(parent)
{
  //! create Gui
  createGui();

  //! connection
  connect(this->addFileButton, SIGNAL(clicked()), this, SLOT(on_addFileButton_clicked()));
  connect(this->removeFileButton, SIGNAL(clicked()), this, SLOT(on_removeFileButton_clicked()));
  connect(this->removeAllFileButton, SIGNAL(clicked()), this, SLOT(on_removeAllFilesButton_clicked()));

  //! default --> no change
  _radioSourceChanged = false;
}


//! ----------- createGui ------------------------------------------------------  
void SettingRadioPage::createGui()
{
    QVBoxLayout* verticalLayout = new QVBoxLayout(this);
    verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
     
    QHBoxLayout* mainHorizontalLayout = new QHBoxLayout();
    mainHorizontalLayout->setObjectName(QString::fromUtf8("mainHorizontalLayout"));
     
    RadioFileList = new QListWidget(this);
    RadioFileList->setObjectName(QString::fromUtf8("RadioFileList"));
    mainHorizontalLayout->addWidget(RadioFileList);
     
    //! Vertical layout for button Remove/Remove All/Add
    QVBoxLayout *verticalLayout_3 = new QVBoxLayout();
    verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
    
    addFileButton = new QPushButton(this);
    addFileButton->setObjectName(QString::fromUtf8("addFileButton"));
    addFileButton->setText(tr("Add ..."));
    
    verticalLayout_3->addWidget(addFileButton);
  
    removeFileButton = new QPushButton(this);
    removeFileButton->setObjectName(QString::fromUtf8("removeFileButton"));
    removeFileButton->setText(tr("Remove"));
  
    verticalLayout_3->addWidget(removeFileButton);
  
    removeAllFileButton = new QPushButton(this);
    removeAllFileButton->setObjectName(QString::fromUtf8("removeAllFileButton"));
    removeAllFileButton->setText(tr("Remove all"));
  
    verticalLayout_3->addWidget(removeAllFileButton);
  
    QSpacerItem* verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);
  
    verticalLayout_3->addItem(verticalSpacer_2);
    mainHorizontalLayout->addLayout(verticalLayout_3);

    verticalLayout->addLayout(mainHorizontalLayout);
}


//! ----------- on_addPathButton_clicked ---------------------------------------  
void SettingRadioPage::on_addFileButton_clicked()
{
     QStringList files = QFileDialog::getOpenFileNames(
		this, 
		tr("Add xspf radio files"), 
		QDesktopServices::storageLocation(QDesktopServices::MusicLocation), 
                tr("Radio files(*.xspf);;All files(*.*)"));

    if(files.isEmpty()) return; 
    QString fileName = files.takeFirst();  
    if (!fileName.isEmpty()) 
    {
      QFile file(fileName);
      if( file.copy(_CONFIGDIR + "/radio/" + QFileInfo(fileName).fileName()) ) {
	//! File copy is OK
        RadioFileList->addItem(QFileInfo(fileName).fileName());	
	_radioSourceChanged = true;
      }
    }
    
}

//! ----------- on_removePathButton_clicked ------------------------------------  
void SettingRadioPage::on_removeFileButton_clicked()
{
    foreach (QListWidgetItem *item, RadioFileList->selectedItems()) {
        QString filename =  item->data(Qt::DisplayRole).toString();
	qDebug() << "SettingRadioPage -  on_removeFileButton_clicked " << filename;
	QFile file(_CONFIGDIR + "/radio/" + filename);
	if(file.exists()) {
	  file.remove();
	  delete RadioFileList->takeItem(RadioFileList->row(item));
	  _radioSourceChanged = true;
	}
    }
}

//! ----------- on_removeAllPathsButton_clicked --------------------------------  
void SettingRadioPage::on_removeAllFilesButton_clicked()
{
    RadioFileList->clear();
    _radioSourceChanged = true;
}
