/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Web/coverthread.h"
#include "Web/coversearch.h"
#include "Database/databasemanager.h"
#include "constants.h"


#include <QSqlField>
#include <QSqlQuery>
#include <QSqlDriver>
#include <QVariant>
#include <QDebug>


//! ----------------------- CoverThread ---------------------------------------
CoverThread::CoverThread(QObject *parent) : QThread(parent) 
{
    setObjectName("coverthread");
}

CoverThread::~CoverThread()
{
}


//! ----------------------- CoverThread::run() ---------------------------------
void CoverThread::run()
{

  qDebug() << "CoverThread::run()";
  _listParam.clear();
  checkDatabase();

  _coverTask = new CoverTask(0);
  _coverTask->_listParam = _listParam;
  connect(_coverTask, SIGNAL(finished()), SLOT(finish()), Qt::QueuedConnection);
  _coverTask->run();

  //! Start thread event loop --> makes signals and slots work
  exec();
  qDebug() << "CoverThread::run() exited";

}

void CoverThread::finish()
{
    qDebug() << "CoverThread::finish";

    delete _coverTask;
    emit coverThreadEnd();
    exit();
}


//! ----------------------- CoverThread::checkDatabase -------------------------
void CoverThread::checkDatabase()
{
  qDebug() << " --- CoverThread -> checkDatabase ";
      
  DatabaseManager dbManager("CoverThread");
  if (dbManager.connectToDB()) {
    //! artist database loop
    QSqlQuery artistsQuery("SELECT id,interpret FROM interprets ORDER BY interpret ASC",
                                       *dbManager.sqlDb());
    while (artistsQuery.next()) 
    {
      //! album database loop
      QSqlQuery albumsQuery("SELECT id,album,cover FROM albums WHERE id IN (SELECT album FROM tracks WHERE interpret="+artistsQuery.value(0).toString()+") ORDER BY year ASC;",
                                          *dbManager.sqlDb());
					  
      while (albumsQuery.next())
      {
	QStringList param = QStringList();
        
        param << artistsQuery.value(1).toString(); // artist
        param << albumsQuery.value(1).toString();  // album
        param << albumsQuery.value(2).toString();  // covername
	    
	QString filePath = _CONFIGDIR + "/albums/" + param[2];
	QFile file(filePath);
	if(!file.exists()) {
	  _listParam.append (param);
	}
      } //! fin while album
    } //! Fin while artist
  
  }
}




CoverTask::CoverTask(QObject *parent) : QObject(parent)
{

}
void CoverTask::run()
{
  qDebug() << "CoverTask::run()";

  processCoverSearch();
}

void CoverTask::processCoverSearch()
{
  if(!_listParam.isEmpty()) {
       QStringList param = _listParam.takeFirst();
       qDebug() << " ---- CoverTask -> param " << param << "\n";
                 
       CoverSearch *_coverSearch = new CoverSearch(param[0], param[1], param[2]);
       connect(_coverSearch, SIGNAL(coverSearchDone(QString)), SLOT(processCoverSearch()));
  }
  else {
      qDebug() << "CoverTask::finished";

     emit finished(); 
  }
}