/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "Widget/searchfield.h"
#include "Widget/spacer.h"
#include <QtGui>


//! ---------- class SearchField -----------------------------------------------
SearchField::SearchField(QWidget *parent) : QLineEdit(parent)
{
  this->setMinimumSize(QSize(200, 26));
  this->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed );
  this->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
  
  _clearButton = new QToolButton(this);
  _clearButton->setIcon(QIcon(":/images/clear-filter.png"));
  _clearButton->setCursor(Qt::ArrowCursor);
  _clearButton->setStyleSheet("QToolButton { border: none; padding: 0px; }");
  _clearButton->hide();
  _clearButton->setToolTip(tr("Clear"));
  _clearButton->setFocusPolicy(Qt::NoFocus);

  _inactiveText = tr("Search in Collection");

  QObject::connect(_clearButton, SIGNAL(clicked()), this, SLOT(clearField()));
  QObject::connect(this, SIGNAL(textChanged(const QString&)), this, SLOT(updateCloseButton(const QString&)));
  QObject::connect(this, SIGNAL(returnPressed()), this, SIGNAL( signalSearchTriggered()));
}

void SearchField::setInactiveText(QString text)
{
  _inactiveText = text;
}


void SearchField::paintEvent(QPaintEvent *event)
{
    if (this->text().isEmpty() && !hasFocus()) {
        QLineEdit::paintEvent(event);
        QStyleOptionFrameV2 panel;
        initStyleOption(&panel);
        QRect r = style()->subElementRect(QStyle::SE_LineEditContents, &panel, this);
        QFontMetrics fm = fontMetrics();
        int horizontalMargin = 2;
        QRect lineRect(horizontalMargin + r.x(), r.y() + (r.height() - fm.height() + 1) / 2,
                       r.width() - 2 * horizontalMargin, fm.height());
        QPainter painter(this);
        painter.setPen(palette().brush(QPalette::Disabled, QPalette::Text).color());
        painter.drawText(lineRect, Qt::AlignLeft|Qt::AlignVCenter, _inactiveText);
    } else {
        QLineEdit::paintEvent(event);
    }
}


void SearchField::clearField() 
{
  clear();
  emit signalSearchTriggered();
}


void SearchField::resizeEvent(QResizeEvent *)
{
      QSize sz = _clearButton->sizeHint();
      int frameWidth = style()->pixelMetric(QStyle::PM_DefaultFrameWidth);
      _clearButton->move(rect().right() - frameWidth - sz.width(),
                        (rect().bottom() + 1 - sz.height())/2);
}

void SearchField::updateCloseButton(const QString& text)
{
     _clearButton->setVisible(!text.isEmpty());
}
