/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QtGui/QApplication>
#include <QTextCodec>
#include <QTranslator>
#include <QNetworkReply>
#include <QDir>
#include <QString>
#include <qtsingleapplication.h>
#include "constants.h"
#include "mainwindow.h"

int main(int argc, char *argv[])
{
    //! QApplication settings
    // QtSingleApplication replace QApplication for one instance
    QtSingleApplication yarock(argc, argv);
    yarock.setApplicationName(APP_NAME);
    yarock.setOrganizationName(ORG_NAME);
    yarock.setApplicationVersion(VERSION);

    //! Locale + Translation
    QString locale = QLocale::system().name().left(2);
    //qDebug() << "== main == locale : " << locale;
      
    QTranslator qtTranslator;
    qtTranslator.load("qt_" + locale,QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    yarock.installTranslator(&qtTranslator);

    QTranslator YaRockTranslator;
    //! avec localDir = /usr/share/locale/YaRock/
    QString localeDir = QLatin1String(LOCALEDIR);
    //qDebug() << "== main == localeDir : " << localeDir;

    // If app was not "installed" use the app directory
    if (!QFile::exists(localeDir)) {
        localeDir = qApp->applicationDirPath() + "/../../translation";
    }

    YaRockTranslator.load(QString("yarock_") + locale,localeDir);
    yarock.installTranslator(&YaRockTranslator);
    
    QTextCodec::setCodecForTr(QTextCodec::codecForName("utf8"));
    QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));
    
    
    //! Check repertoire de CONFIG
    QDir *dir = new QDir(_CONFIGDIR);

    if(dir->exists(_CONFIGDIR)) {
          qDebug() << "repertoire config existe : " << _CONFIGDIR;
    }
    else {
      qDebug() << "repertoire config n'existe pas : ";
      QDir().mkpath(_CONFIGDIR);
    }
    
    //! Type registration  
    // -->  This is required in order to use QNetworkReply::NetworkError in QueuedConnetions
    qRegisterMetaType<QNetworkReply::NetworkError>("QNetworkReply::NetworkError");
    
    //! Lancement Mainwindow
    MainWindow         mainwindow;
    mainwindow.show();

    return yarock.exec();
}
