/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _MAINWINDOW_H
#define _MAINWINDOW_H

#include "Player/player.h"
#include "Playlist/playlistview.h"
#include "Playlist/playlistmodel.h"
#include "Playlist/playlistwidget.h"
#include "NowPlaying/nowplayingview.h"

#include "CollectionModel/collectionpopulator.h"
#include "CollectionModel/plsfilepopulator.h"
#include "CollectionView/collectionscene.h"
#include "CollectionView/collectionwidget.h"
#include "CollectionView/virtualplaylist.h"

#include "CentralWidget/centraltoolbar.h"
#include "Widget/playertoolbar.h"
#include "Widget/statusbar.h"

#include "CentralWidget/centralwidget.h"
#include "CentralWidget/menuview.h"
#include "Web/coverthread.h"

#include "threadpool.h"

#include "RadioModel/streammodel.h"
#include "RadioView/radioscene.h"
#include "RadioView/radiowidget.h"

#include <QMainWindow>
#include <QtGui>
#include <QtCore>
   
   
class MainWindow : public QMainWindow 
{
    Q_OBJECT
public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();
    
private slots:
    //! ------------- MainWindows  ---------------------------------------------
    void createGui();
    void createActions();
    void createMenuBar();
    void connectSlots();
    void createSystray();
    //! ------------- File management ------------------------------------------
    void addFileToPlaylist();
    void addDirToPlaylist();
    void addUrlToPlaylist();
    //! ------------- Playlist management --------------------------------------
    void clearPlaylist();
    void savePlaylist();
    void autoSavePlaylist();
    void updatePlaylist(QList<QUrl> listUrl);
    //! ------------- Player management ----------------------------------------
    void playOrPause();
    void stopPlayer();
    void playNext();
    void playPrev();
    void playerStateChanged(Phonon::State newState, Phonon::State oldState);
    void playTrackFromPlaylist();
    void playTrackFromCollection();  
    //! ------------- Settings Method ------------------------------------------
    void openCollectionSettingsDlg();
    void openAllSettingsDlg();
    void openRadioSettingsDlg();
    //! ------------- Database Method ------------------------------------------
    void firstStartDialog();
    void rebuildDatabase();
    void forceRebuildDatabaseDlg();
    void forceRebuildDatabase();
    void populateCollectionModel();
    void populatePlsFileModel();
    void updateCollectionScene();
    void updatePlsFileScene();
    void checkNewPlaylistSaved(QString);
    //! ------------- Radio Method ---------------------------------------------
    void rebuildRadio();
    //! ------------- Cover Search Thread --------------------------------------
    void startCoverSearchThread();
    void stopCoverSearchThread();
    //! ------------- Others Actions -------------------------------------------
    void on_actionAbout_YaRock_triggered();
    void on_actionAbout_Qt_triggered();
    void on_actionQuit_Yarock_triggered();
    //! ------------- Systray --------------------------------------------------
    void trayClicked(QSystemTrayIcon::ActivationReason);
	

protected:
    void closeEvent(QCloseEvent *event);
      	
private:
    enum AppState {Closing, Running};
    AppState              _state;
        
    Player                *_player;
    PlayerToolBar         *_playerToolBar;
    StatusBar             *_statusBar;

    CentralToolBar        *_centralToolBar;
    CentralWidget         *_centralWidget;
    MenuView              *_menuView;
    MenuModel             *_menuModel;

    PlaylistView          *_playlistView;
    PlaylistModel         *_playlistModel;
    PlaylistWidget        *_playlistwidget;
    NowPlayingView        *_nowplayingview;
    FileViewer            *_fileViewerWidget;
  
    CollectionModel       *_collectionModel;
    PlsFileModel          *_plsFileModel;
    CollectionScene       *_collectionScene;
    CollectionPopulator   *_collectionPopulator;
    PlsFilePopulator      *_plsFilePopulator;
    CollectionWidget      *_collectionWidget;

    VirtualPlaylist       *_virtualPlaylist;
      
    StreamModel           *_streamModel;
    RadioScene            *_radioScene;
    RadioWidget           *_radioWidget;    
    
    CoverThread           *_coverThread;
    ThreadPool            *_threadPool;
    
    QMap<QString, QAction*>  *actions;
  
    QSettings             *_settings;
    QIcon                 *_appIcon;
    QSystemTrayIcon       *_trayIcon;
    bool                  _canClose;
    bool                  _systrayActivated;    
};

#endif // _MAINWINDOW_H
