/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "threadpool.h"
#include "CollectionModel/collectionpopulator.h"
#include "CollectionModel/plsfilepopulator.h"
#include "Database/databasebuilder.h"
#include "Database/databaseplsbuilder.h"
#include "Playlist/playlistpopulator.h"
#include "Playlist/playlistwriter.h"

#include <QDir>
#include <QSettings>
#include <QStringList>
#include <QUrl>

//! ThreadPool construction ----------------------------------------------------
ThreadPool::ThreadPool()
{
    _threadPool          = new QThreadPool(this);
    
    _playlistPopulator   = new PlaylistPopulator();
    _playlistPopulator->setAutoDelete(false);

    _playlistWriter      = new PlaylistWriter();
    _playlistWriter->setAutoDelete(false);    
    QObject::connect(_playlistWriter,SIGNAL(playlistSaved(QString)),this,SIGNAL(playlistSaved(QString)));

    _databaseBuilder = new DataBaseBuilder();
    _databaseBuilder->setAutoDelete(false);    
    QObject::connect(_databaseBuilder,SIGNAL(buildingFinished()),this,SIGNAL(databaseBuild()));

    _databasePlsBuilder = new DataBasePlsBuilder();
    _databasePlsBuilder->setAutoDelete(false);    
    QObject::connect(_databasePlsBuilder,SIGNAL(buildingFinished()),this,SIGNAL(databasePlsBuild()));
    
    _collectionPopulator = new CollectionPopulator();
    _collectionPopulator->setAutoDelete(false);
    QObject::connect(_collectionPopulator,SIGNAL(modelPopulated()),this,SIGNAL(collectionModelPopulated()));

    _plsFilePopulator = new PlsFilePopulator();
    _plsFilePopulator->setAutoDelete(false);    
    QObject::connect(_plsFilePopulator,SIGNAL(modelPopulated()),this,SIGNAL(plsFileModelPopulated()));
}

ThreadPool::~ThreadPool()
{
    _threadPool->waitForDone();
    delete  _playlistPopulator;
    delete  _playlistWriter;
    delete _databaseBuilder;
    delete _databasePlsBuilder;
    delete _collectionPopulator;
    delete _plsFilePopulator;
    delete _threadPool;
}

//! Playlist Populator Thread --------------------------------------------------
void ThreadPool::PlaylistAddFiles(const QStringList &files) 
{
  if(_playlistPopulator->isRunning()) return;
  _playlistPopulator->addFiles(files);
  _threadPool->start(_playlistPopulator);
}

void ThreadPool::PlaylistAddFile(const QString &file) 
{
  if(_playlistPopulator->isRunning()) return;
  _playlistPopulator->addFile(file);
  _threadPool->start (_playlistPopulator);
}

void ThreadPool::PlaylistAddUrls(QList<QUrl> listUrl) 
{
  if(_playlistPopulator->isRunning()) return;
  _playlistPopulator->addUrls(listUrl);
  _threadPool->start (_playlistPopulator);
}


//! Playlist Writer Thread -----------------------------------------------------
void ThreadPool::PlaylistSaveToFile(const QString &filename)
{
  if(_playlistWriter->isRunning()) return;
  _playlistWriter->saveToFile(filename);
  _threadPool->start (_playlistWriter);
}

//! Database Thread ------------------------------------------------------------
void ThreadPool::DatabaseBuild(QStringList listDir)
{
  if(_databaseBuilder->isRunning()) return;
  _databaseBuilder->rebuildFolder(listDir);
  _threadPool->start (_databaseBuilder);
}

void ThreadPool::DatabasePlsBuild(QStringList listDir)
{
  if(_databasePlsBuilder->isRunning()) return;
  _databasePlsBuilder->rebuildFolder(listDir);
  _threadPool->start (_databasePlsBuilder);
}

void ThreadPool::DatabasePlsBuildInternal()
{
  if(_databasePlsBuilder->isRunning()) return;
  _databasePlsBuilder->rebuildOnlyInternalFolder();
  _threadPool->start (_databasePlsBuilder);
}


//! Model Populator Thread -----------------------------------------------------
void ThreadPool::PopulateCollectionModel()
{
 if(_collectionPopulator->isRunning()) return;
  _threadPool->start (_collectionPopulator);
}

void ThreadPool::PopulatePlsFileModel()
{
 if(_plsFilePopulator->isRunning()) return;
  _threadPool->start (_plsFilePopulator);
}
