/* YaRock Player
* Copyright (C) 2010 Sebastien amardeilh (sebastien.amardeilh@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _THREADPOOL_H_
#define _THREADPOOL_H_

#include <QObject>
#include <QThreadPool>
#include <QList>
#include <QUrl>
#include <QString>
#include <QStringList>

//! thread class
class DataBaseBuilder;      //! thread for building database for music files
class DataBasePlsBuilder;   //! thread for building database for playlist

class CollectionPopulator;  //! thread to populate CollectionModel 
class PlsFilePopulator;     //! thread to populate PlsFileModel
class CoverThread;          //! thread to search cover art from lastfm

class PlaylistPopulator;    //! thread to populate playlist
class PlaylistWriter;       //! thread to save playlist to file

class ThreadPool : public QObject
{
Q_OBJECT
  public:
    explicit ThreadPool();
    ~ThreadPool();
    
  public slots:
    //! Playlist Populator Thread
    void PlaylistAddFiles(const QStringList &files);
    void PlaylistAddFile(const QString &file);
    void PlaylistAddUrls(QList<QUrl> listUrl);

    //! Playlist Writer Thread
    void PlaylistSaveToFile(const QString &filename);
    
    //! Database Builder Thread
    void DatabaseBuild(QStringList listDir);
    void DatabasePlsBuild(QStringList listDir);
    void DatabasePlsBuildInternal();
    
    //! Model Populator Thread
    void PopulateCollectionModel();
    void PopulatePlsFileModel();

    //! Cover Search Thread 

  private:
    QThreadPool            *_threadPool;
    PlaylistPopulator      *_playlistPopulator;
    PlaylistWriter         *_playlistWriter; 
    DataBaseBuilder        *_databaseBuilder;
    DataBasePlsBuilder     *_databasePlsBuilder;
    CollectionPopulator    *_collectionPopulator;
    PlsFilePopulator       *_plsFilePopulator;
    
  signals:
    void playlistSaved(QString);
    void databaseBuild();
    void databasePlsBuild();
    void collectionModelPopulated();
    void plsFileModelPopulated();
};

#endif // _THREADPOOL_H_
